@@ -41,7 +41,7 @@
       "It can be used only by extensions.gnome.org"
 #define PLUGIN_MIME_STRING "application/x-gnome-shell-integration::Gnome Shell Integration Dummy Content-Type";
 
-#define PLUGIN_API_VERSION 3
+#define PLUGIN_API_VERSION 4
 
 typedef struct {
   GDBusProxy *proxy;
@@ -163,6 +163,7 @@ NP_Initialize(NPNetscapeFuncs *pfuncs, NPPluginFuncs *plugin)
   plugin->newp = NPP_New;
   plugin->destroy = NPP_Destroy;
   plugin->getvalue = NPP_GetValue;
+  plugin->setwindow = NPP_SetWindow;
 
   return NPERR_NO_ERROR;
 }
@@ -267,6 +268,7 @@ typedef struct {
   NPObject     parent;
   NPP          instance;
   GDBusProxy  *proxy;
+  GSettings   *settings;
   NPObject    *listener;
   NPObject    *restart_listener;
   gint         signal_id;
@@ -323,6 +325,9 @@ on_shell_appeared (GDBusConnection *connection,
     }
 }
 
+#define SHELL_SCHEMA "org.gnome.shell"
+#define ENABLED_EXTENSIONS_KEY "enabled-extensions"
+
 static NPObject *
 plugin_object_allocate (NPP      instance,
                         NPClass *klass)
@@ -332,6 +337,7 @@ plugin_object_allocate (NPP      instance,
 
   obj->instance = instance;
   obj->proxy = g_object_ref (data->proxy);
+  obj->settings = g_settings_new (SHELL_SCHEMA);
   obj->signal_id = g_signal_connect (obj->proxy, "g-signal",
                                      G_CALLBACK (on_shell_signal), obj);
 
@@ -492,7 +498,61 @@ plugin_enable_extension (PluginObject *obj,
                          NPString      uuid,
                          gboolean      enabled)
 {
+  gboolean ret;
   gchar *uuid_str = g_strndup (uuid.UTF8Characters, uuid.UTF8Length);
+  gsize length;
+  gchar **uuids;
+  const gchar **new_uuids;
+
+  if (!uuid_is_valid (uuid_str))
+    {
+      g_free (uuid_str);
+      return FALSE;
+    }
+
+  uuids = g_settings_get_strv (obj->settings, ENABLED_EXTENSIONS_KEY);
+  length = g_strv_length (uuids);
+
+  if (enabled)
+    {
+      new_uuids = g_new (const gchar *, length + 2); /* New key, NULL */
+      memcpy (new_uuids, uuids, length * sizeof (*new_uuids));
+      new_uuids[length] = uuid_str;
+      new_uuids[length + 1] = NULL;
+    }
+  else
+    {
+      gsize i = 0, j = 0;
+      new_uuids = g_new (const gchar *, length);
+      for (i = 0; i < length; i ++)
+        {
+          if (g_str_equal (uuids[i], uuid_str))
+            continue;
+
+          new_uuids[j] = uuids[i];
+          j++;
+        }
+
+      new_uuids[j] = NULL;
+    }
+
+  ret = g_settings_set_strv (obj->settings,
+                             ENABLED_EXTENSIONS_KEY,
+                             new_uuids);
+
+  g_strfreev (uuids);
+  g_free (new_uuids);
+  g_free (uuid_str);
+
+  return ret;
+}
+
+static gboolean
+plugin_install_extension (PluginObject *obj,
+                          NPString      uuid)
+{
+  gchar *uuid_str = g_strndup (uuid.UTF8Characters, uuid.UTF8Length);
+
   if (!uuid_is_valid (uuid_str))
     {
       g_free (uuid_str);
@@ -500,7 +560,7 @@ plugin_enable_extension (PluginObject *obj,
     }
 
   g_dbus_proxy_call (obj->proxy,
-                     (enabled ? "EnableExtension" : "DisableExtension"),
+                     "InstallRemoteExtension",
                      g_variant_new ("(s)", uuid_str),
                      G_DBUS_CALL_FLAGS_NONE,
                      -1, /* timeout */
@@ -513,40 +573,6 @@ plugin_enable_extension (PluginObject *obj,
   return TRUE;
 }
 
-static gboolean
-plugin_install_extension (PluginObject *obj,
-                          NPString      uuid,
-                          NPString      version_tag)
-{
-  gchar *uuid_str = g_strndup (uuid.UTF8Characters, uuid.UTF8Length);
-  gchar *version_tag_str;
-
-  if (!uuid_is_valid (uuid_str))
-    {
-      g_free (uuid_str);
-      return FALSE;
-    }
-
-  version_tag_str = g_strndup (version_tag.UTF8Characters,
-                               version_tag.UTF8Length);
-
-  g_dbus_proxy_call (obj->proxy,
-                     "InstallRemoteExtension",
-                     g_variant_new ("(ss)",
-                                    uuid_str,
-                                    version_tag_str),
-                     G_DBUS_CALL_FLAGS_NONE,
-                     -1, /* timeout */
-                     NULL, /* cancellable */
-                     NULL, /* callback */
-                     NULL /* user_data */);
-
-  g_free (uuid_str);
-  g_free (version_tag_str);
-
-  return TRUE;
-}
-
 static gboolean
 plugin_uninstall_extension (PluginObject *obj,
                             NPString      uuid,
@@ -771,11 +797,9 @@ plugin_object_invoke (NPObject        *npobj,
   else if (name == install_extension_id)
     {
       if (!NPVARIANT_IS_STRING(args[0])) return FALSE;
-      if (!NPVARIANT_IS_STRING(args[1])) return FALSE;
 
       return plugin_install_extension (obj,
-                                       NPVARIANT_TO_STRING(args[0]),
-                                       NPVARIANT_TO_STRING(args[1]));
+                                       NPVARIANT_TO_STRING(args[0]));
     }
   else if (name == uninstall_extension_id)
     {
@@ -946,3 +970,12 @@ NPP_GetValue(NPP          instance,
 
   return NPERR_NO_ERROR;
 }
+
+/* Opera tries to call NPP_SetWindow without checking the
+ * NULL pointer beforehand. */
+NPError
+NPP_SetWindow(NPP          instance,
+              NPWindow    *window)
+{
+  return NPERR_NO_ERROR;
+}
