@@ -40,13 +40,18 @@ function getCurrentExtension() {
         throw new Error('Could not find current extension');
 
     let path = match[1];
-    let uuid = GLib.path_get_basename(GLib.path_get_dirname(path));
+    let file = Gio.File.new_for_path(path);
 
-    let extension = extensions[uuid];
-    if (extension === undefined)
-        throw new Error('Could not find current extension');
+    // Walk up the directory tree, looking for an extesion with
+    // the same UUID as a directory name.
+    while (file != null) {
+        let extension = extensions[file.get_basename()];
+        if (extension !== undefined)
+            return extension;
+        file = file.get_parent();
+    }
 
-    return extension;
+    throw new Error('Could not find current extension');
 }
 
 /**
@@ -120,7 +125,7 @@ function createExtensionObject(uuid, dir, type) {
 
     // Encourage people to add this
     if (!meta.url) {
-        global.log('Warning: Missing "url" property in metadata.json');
+        log('Warning: Missing "url" property in %s/metadata.json'.format(uuid));
     }
 
     if (uuid != meta.uuid) {
@@ -157,7 +162,7 @@ function init() {
         if (!userExtensionsDir.query_exists(null))
             userExtensionsDir.make_directory_with_parents(null);
     } catch (e) {
-        global.logError('' + e);
+        logError(e, 'Could not create extensions directory');
     }
 }
 
@@ -167,7 +172,7 @@ function scanExtensionsInDirectory(callback, dir, type) {
     try {
         fileEnum = dir.enumerate_children('standard::*', Gio.FileQueryInfoFlags.NONE, null);
     } catch(e) {
-        global.logError('' + e);
+        logError(e, 'Could not enumerate extensions directory');
         return;
     }
 
