@@ -312,11 +312,10 @@ const AppSearchProvider = new Lang.Class({
 
     _init: function() {
         this.parent(_("APPLICATIONS"));
-
         this._appSys = Shell.AppSystem.get_default();
     },
 
-    getResultMetas: function(apps) {
+    getResultMetas: function(apps, callback) {
         let metas = [];
         for (let i = 0; i < apps.length; i++) {
             let app = apps[i];
@@ -327,15 +326,15 @@ const AppSearchProvider = new Lang.Class({
                          }
                        });
         }
-        return metas;
+        callback(metas);
     },
 
     getInitialResultSet: function(terms) {
-        return this._appSys.initial_search(terms);
+        this.searchSystem.pushResults(this, this._appSys.initial_search(terms));
     },
 
     getSubsearchResultSet: function(previousResults, terms) {
-        return this._appSys.subsearch(previousResults, terms);
+        this.searchSystem.pushResults(this, this._appSys.subsearch(previousResults, terms));
     },
 
     activateResult: function(app, params) {
@@ -378,7 +377,7 @@ const SettingsSearchProvider = new Lang.Class({
         this._gnomecc = this._appSys.lookup_app('gnome-control-center.desktop');
     },
 
-    getResultMetas: function(prefs) {
+    getResultMetas: function(prefs, callback) {
         let metas = [];
         for (let i = 0; i < prefs.length; i++) {
             let pref = prefs[i];
@@ -389,15 +388,15 @@ const SettingsSearchProvider = new Lang.Class({
                          }
                        });
         }
-        return metas;
+        callback(metas);
     },
 
     getInitialResultSet: function(terms) {
-        return this._appSys.search_settings(terms);
+        this.searchSystem.pushResults(this, this._appSys.search_settings(terms));
     },
 
     getSubsearchResultSet: function(previousResults, terms) {
-        return this._appSys.search_settings(terms);
+        this.searchSystem.pushResults(this, this._appSys.search_settings(terms));
     },
 
     activateResult: function(pref, params) {
