@@ -34,24 +34,11 @@ const REPOSITORY_URL_BASE = 'https://extensions.gnome.org';
 const REPOSITORY_URL_DOWNLOAD = REPOSITORY_URL_BASE + '/download-extension/%s.shell-extension.zip';
 const REPOSITORY_URL_INFO =     REPOSITORY_URL_BASE + '/extension-info/';
 
-const _httpSession = new Soup.SessionAsync();
+const _httpSession = new Soup.SessionAsync({ ssl_use_system_ca_file: true });
 
-// The unfortunate state of gjs, gobject-introspection and libsoup
-// means that I have to do a hack to add a feature.
 // See: https://bugzilla.gnome.org/show_bug.cgi?id=655189 for context.
-
-if (Soup.Session.prototype.add_feature != null)
-    Soup.Session.prototype.add_feature.call(_httpSession, new Soup.ProxyResolverDefault());
-
-function _getCertFile() {
-    let localCert = GLib.build_filenamev([global.userdatadir, 'extensions.gnome.org.crt']);
-    if (GLib.file_test(localCert, GLib.FileTest.EXISTS))
-        return localCert;
-    else
-        return Config.SHELL_SYSTEM_CA_FILE;
-}
-
-_httpSession.ssl_ca_file = _getCertFile();
+// _httpSession.add_feature(new Soup.ProxyResolverDefault());
+Soup.Session.prototype.add_feature.call(_httpSession, new Soup.ProxyResolverDefault());
 
 // Arrays of uuids
 var enabledExtensions;
@@ -69,18 +56,16 @@ const disconnect = Lang.bind(_signals, _signals.disconnect);
 
 const ENABLED_EXTENSIONS_KEY = 'enabled-extensions';
 
-function installExtensionFromUUID(uuid, version_tag) {
+function installExtensionFromUUID(uuid) {
     let params = { uuid: uuid,
-                   version_tag: version_tag,
-                   shell_version: Config.PACKAGE_VERSION,
-                   api_version: API_VERSION.toString() };
+                   shell_version: Config.PACKAGE_VERSION };
 
     let message = Soup.form_request_new_from_hash('GET', REPOSITORY_URL_INFO, params);
 
     _httpSession.queue_message(message,
                                function(session, message) {
                                    let info = JSON.parse(message.response_body.data);
-                                   let dialog = new InstallExtensionDialog(uuid, version_tag, info.name);
+                                   let dialog = new InstallExtensionDialog(uuid, info);
                                    dialog.open(global.get_current_time());
                                });
 }
@@ -115,21 +100,13 @@ function gotExtensionZipFile(session, message, uuid) {
         return;
     }
 
-    // FIXME: use a GFile mkstemp-type method once one exists
-    let fd, tmpzip;
-    try {
-        [fd, tmpzip] = GLib.file_open_tmp('XXXXXX.shell-extension.zip');
-    } catch (e) {
-        logExtensionError(uuid, 'tempfile: ' + e.toString());
-        return;
-    }
-
-    let stream = new Gio.UnixOutputStream({ fd: fd });
+    let [file, stream] = Gio.File.new_tmp('XXXXXX.shell-extension.zip');
     let dir = ExtensionUtils.userExtensionsDir.get_child(uuid);
-    Shell.write_soup_message_to_stream(stream, message);
+    let contents = message.response_body.flatten().as_bytes();
+    stream.output_stream.write_bytes(contents, null);
     stream.close(null);
     let [success, pid] = GLib.spawn_async(null,
-                                          ['unzip', '-uod', dir.get_path(), '--', tmpzip],
+                                          ['unzip', '-uod', dir.get_path(), '--', file.get_path()],
                                           null,
                                           GLib.SpawnFlags.SEARCH_PATH | GLib.SpawnFlags.DO_NOT_REAP_CHILD,
                                           null);
@@ -237,7 +214,7 @@ function logExtensionError(uuid, message, state) {
         extension.errors = [];
 
     extension.errors.push(message);
-    global.logError('Extension "%s" had error: %s'.format(uuid, message));
+    log('Extension "%s" had error: %s'.format(uuid, message));
     state = state || ExtensionState.ERROR;
     _signals.emit('extension-state-changed', { uuid: uuid,
                                                error: message,
@@ -249,7 +226,7 @@ function loadExtension(dir, type, enabled) {
     let extension;
 
     if (ExtensionUtils.extensions[uuid] != undefined) {
-        global.logError('Extension "%s" is already loaded'.format(uuid));
+        log('Extension "%s" is already loaded'.format(uuid));
         return;
     }
 
@@ -278,7 +255,6 @@ function loadExtension(dir, type, enabled) {
     }
 
     _signals.emit('extension-state-changed', extension);
-    global.log('Loaded extension ' + uuid);
 }
 
 function initExtension(uuid) {
@@ -390,12 +366,11 @@ const InstallExtensionDialog = new Lang.Class({
     Name: 'InstallExtensionDialog',
     Extends: ModalDialog.ModalDialog,
 
-    _init: function(uuid, version_tag, name) {
+    _init: function(uuid, info) {
         this.parent({ styleClass: 'extension-dialog' });
 
         this._uuid = uuid;
-        this._version_tag = version_tag;
-        this._name = name;
+        this._info = info;
 
         this.setButtons([{ label: _("Cancel"),
                            action: Lang.bind(this, this._onCancelButtonPressed),
@@ -405,13 +380,17 @@ const InstallExtensionDialog = new Lang.Class({
                            action: Lang.bind(this, this._onInstallButtonPressed)
                          }]);
 
-        let message = _("Download and install '%s' from extensions.gnome.org?").format(name);
+        let message = _("Download and install '%s' from extensions.gnome.org?").format(info.name);
 
-        this._descriptionLabel = new St.Label({ text: message });
+        let box = new St.BoxLayout();
+        this.contentLayout.add(box);
 
-        this.contentLayout.add(this._descriptionLabel,
-                               { y_fill:  true,
-                                 y_align: St.Align.START });
+        let gicon = new Gio.FileIcon({ file: Gio.File.new_for_uri(REPOSITORY_URL_BASE + info.icon) })
+        let icon = new St.Icon({ gicon: gicon });
+        box.add(icon);
+
+        let label = new St.Label({ text: message });
+        box.add(label);
     },
 
     _onCancelButtonPressed: function(button, event) {
@@ -435,9 +414,7 @@ const InstallExtensionDialog = new Lang.Class({
 
         _signals.emit('extension-state-changed', state);
 
-        let params = { version_tag: this._version_tag,
-                       shell_version: Config.PACKAGE_VERSION,
-                       api_version: API_VERSION.toString() };
+        let params = { shell_version: Config.PACKAGE_VERSION };
 
         let url = REPOSITORY_URL_DOWNLOAD.format(this._uuid);
         let message = Soup.form_request_new_from_hash('GET', url, params);
