@@ -30,6 +30,7 @@ const NetworkAgent = imports.ui.networkAgent;
 const NotificationDaemon = imports.ui.notificationDaemon;
 const WindowAttentionHandler = imports.ui.windowAttentionHandler;
 const Scripting = imports.ui.scripting;
+const SessionMode = imports.ui.sessionMode;
 const ShellDBus = imports.ui.shellDBus;
 const TelepathyClient = imports.ui.telepathyClient;
 const WindowManager = imports.ui.windowManager;
@@ -46,7 +47,6 @@ let automountManager = null;
 let autorunManager = null;
 let panel = null;
 let hotCorners = [];
-let placesManager = null;
 let overview = null;
 let runDialog = null;
 let lookingGlass = null;
@@ -57,6 +57,7 @@ let windowAttentionHandler = null;
 let telepathyClient = null;
 let ctrlAltTabManager = null;
 let recorder = null;
+let sessionMode = null;
 let shellDBusService = null;
 let modalCount = 0;
 let modalActorFocusStack = [];
@@ -67,28 +68,27 @@ let statusIconDispatcher = null;
 let keyboard = null;
 let layoutManager = null;
 let networkAgent = null;
-let _errorLogStack = [];
 let _startDate;
 let _defaultCssStylesheet = null;
 let _cssStylesheet = null;
-let _gdmCssStylesheet = null;
 let _overridesSettings = null;
 
 let background = null;
 
-function _createUserSession() {
+function createUserSession() {
     // Load the calendar server. Note that we are careful about
     // not loading any events until the user presses the clock
     global.launch_calendar_server();
 
-    placesManager = new PlaceDisplay.PlacesManager();
     telepathyClient = new TelepathyClient.Client();
     automountManager = new AutomountManager.AutomountManager();
     autorunManager = new AutorunManager.AutorunManager();
     networkAgent = new NetworkAgent.NetworkAgent();
+
+    _initRecorder();
 }
 
-function _createGDMSession() {
+function createGDMSession() {
     // We do this this here instead of at the top to prevent GDM
     // related code from getting loaded in normal user sessions
     const LoginDialog = imports.gdm.loginDialog;
@@ -99,18 +99,26 @@ function _createGDMSession() {
                         });
 }
 
+function createInitialSetupSession() {
+    networkAgent = new NetworkAgent.NetworkAgent();
+}
+
 function _initRecorder() {
     let recorderSettings = new Gio.Settings({ schema: 'org.gnome.shell.recorder' });
+    let desktopLockdownSettings = new Gio.Settings({ schema: 'org.gnome.desktop.lockdown' });
+    let bindingSettings = new Gio.Settings({ schema: 'org.gnome.shell.keybindings' });
 
-    global.screen.connect('toggle-recording', function() {
+    global.display.add_keybinding('toggle-recording',
+                                  bindingSettings,
+                                  Meta.KeyBindingFlags.NONE, function() {
         if (recorder == null) {
             recorder = new Shell.Recorder({ stage: global.stage });
         }
 
         if (recorder.is_recording()) {
-            recorder.pause();
+            recorder.close();
             Meta.enable_unredirect_for_screen(global.screen);
-        } else {
+        } else if (!desktopLockdownSettings.get_boolean('disable-save-to-disk')) {
             // read the parameters from GSettings always in case they have changed
             recorder.set_framerate(recorderSettings.get_int('framerate'));
             /* Translators: this is a filename used for screencast recording */
@@ -129,38 +137,17 @@ function _initRecorder() {
     });
 }
 
-function _initUserSession() {
-    _initRecorder();
-
-    global.screen.override_workspace_layout(Meta.ScreenCorner.TOPLEFT, false, -1, 1);
-
-    ExtensionSystem.init();
-    ExtensionSystem.loadExtensions();
-
-    Meta.keybindings_set_custom_handler('panel-run-dialog', function() {
-       getRunDialog().open();
-    });
-
-    Meta.keybindings_set_custom_handler('panel-main-menu', function () {
-        overview.toggle();
-    });
-
-    global.display.connect('overlay-key', Lang.bind(overview, overview.toggle));
-
-}
-
 function start() {
-    // Monkey patch utility functions into the global proxy;
-    // This is easier and faster than indirecting down into global
-    // if we want to call back up into JS.
-    global.logError = _logError;
-    global.log = _logDebug;
+    // These are here so we don't break compatibility.
+    global.logError = window.log;
+    global.log = window.log;
 
     // Chain up async errors reported from C
     global.connect('notify-error', function (global, msg, detail) { notifyError(msg, detail); });
 
     Gio.DesktopAppInfo.set_desktop_env('GNOME');
 
+    sessionMode = new SessionMode.SessionMode();
     shellDBusService = new ShellDBus.GnomeShell();
 
     // Ensure ShellWindowTracker and ShellAppUsage are initialized; this will
@@ -183,7 +170,6 @@ function start() {
     global.stage.no_clear_hint = true;
 
     _defaultCssStylesheet = global.datadir + '/theme/gnome-shell.css';
-    _gdmCssStylesheet = global.datadir + '/theme/gdm.css';
     loadTheme();
 
     // Set up stage hierarchy to group all UI actors under one container.
@@ -211,8 +197,7 @@ function start() {
     layoutManager = new Layout.LayoutManager();
     xdndHandler = new XdndHandler.XdndHandler();
     ctrlAltTabManager = new CtrlAltTab.CtrlAltTabManager();
-    // This overview object is just a stub for non-user sessions
-    overview = new Overview.Overview({ isDummy: global.session_type != Shell.SessionType.USER });
+    overview = new Overview.Overview();
     magnifier = new Magnifier.Magnifier();
     statusIconDispatcher = new StatusIconDispatcher.StatusIconDispatcher();
     panel = new Panel.Panel();
@@ -222,10 +207,7 @@ function start() {
     notificationDaemon = new NotificationDaemon.NotificationDaemon();
     windowAttentionHandler = new WindowAttentionHandler.WindowAttentionHandler();
 
-    if (global.session_type == Shell.SessionType.USER)
-        _createUserSession();
-    else if (global.session_type == Shell.SessionType.GDM)
-        _createGDMSession();
+    sessionMode.createSession();
 
     panel.startStatusArea();
 
@@ -233,8 +215,30 @@ function start() {
     keyboard.init();
     overview.init();
 
-    if (global.session_type == Shell.SessionType.USER)
-        _initUserSession();
+    if (sessionMode.hasWorkspaces)
+        global.screen.override_workspace_layout(Meta.ScreenCorner.TOPLEFT,
+                                                false, -1, 1);
+
+    if (sessionMode.allowExtensions) {
+        ExtensionSystem.init();
+        ExtensionSystem.loadExtensions();
+    }
+
+    if (sessionMode.hasRunDialog) {
+        Meta.keybindings_set_custom_handler('panel-run-dialog', function() {
+           getRunDialog().open();
+        });
+    }
+
+    if (sessionMode.hasOverview) {
+        Meta.keybindings_set_custom_handler('panel-main-menu', function () {
+            overview.toggle();
+        });
+
+        global.display.connect('overlay-key',
+                               Lang.bind(overview, overview.toggle));
+    }
+
     statusIconDispatcher.start(messageTray.actor);
 
     // Provide the bus object for gnome-session to
@@ -251,7 +255,6 @@ function start() {
 
     global.stage.connect('captured-event', _globalKeyPressHandler);
 
-    _log('info', 'loaded at ' + _startDate);
     log('GNOME Shell started at ' + _startDate);
 
     let perfModuleName = GLib.getenv("SHELL_PERF_MODULE");
@@ -494,8 +497,8 @@ function loadTheme() {
 
     let theme = new St.Theme ({ application_stylesheet: cssStylesheet });
 
-    if (global.session_type == Shell.SessionType.GDM)
-        theme.load_stylesheet(_gdmCssStylesheet);
+    if (sessionMode.extraStylesheet)
+        theme.load_stylesheet(sessionMode.extraStylesheet);
 
     if (previousTheme) {
         let customStylesheets = previousTheme.get_custom_stylesheets();
@@ -537,59 +540,6 @@ function notifyError(msg, details) {
     notify(msg, details);
 }
 
-/**
- * _log:
- * @category: string message type ('info', 'error')
- * @msg: A message string
- * ...: Any further arguments are converted into JSON notation,
- *      and appended to the log message, separated by spaces.
- *
- * Log a message into the LookingGlass error
- * stream.  This is primarily intended for use by the
- * extension system as well as debugging.
- */
-function _log(category, msg) {
-    let text = msg;
-    if (arguments.length > 2) {
-        text += ': ';
-        for (let i = 2; i < arguments.length; i++) {
-            text += JSON.stringify(arguments[i]);
-            if (i < arguments.length - 1)
-                text += ' ';
-        }
-    }
-    _errorLogStack.push({timestamp: new Date().getTime(),
-                         category: category,
-                         message: text });
-}
-
-function _logError(msg) {
-    return _log('error', msg);
-}
-
-function _logDebug(msg) {
-    return _log('debug', msg);
-}
-
-// Used by the error display in lookingGlass.js
-function _getAndClearErrorStack() {
-    let errors = _errorLogStack;
-    _errorLogStack = [];
-    return errors;
-}
-
-function logStackTrace(msg) {
-    try {
-        throw new Error();
-    } catch (e) {
-        // e.stack must have at least two lines, with the first being
-        // logStackTrace() (which we strip off), and the second being
-        // our caller.
-        let trace = e.stack.substr(e.stack.indexOf('\n') + 1);
-        log(msg ? (msg + '\n' + trace) : trace);
-    }
-}
-
 function isWindowActorDisplayedOnWorkspace(win, workspaceIndex) {
     return win.get_workspace() == workspaceIndex ||
         (win.get_meta_window() && win.get_meta_window().is_on_all_workspaces());
@@ -612,6 +562,11 @@ function _globalKeyPressHandler(actor, event) {
     if (event.type() != Clutter.EventType.KEY_PRESS)
         return false;
 
+    if (!sessionMode.allowKeybindingsWhenModal) {
+        if (modalCount > (overview.visible ? 1 : 0))
+            return false;
+    }
+
     let symbol = event.get_key_symbol();
     let keyCode = event.get_key_code();
     let ignoredModifiers = global.display.get_ignored_modifier_mask();
@@ -620,11 +575,6 @@ function _globalKeyPressHandler(actor, event) {
     // This relies on the fact that Clutter.ModifierType is the same as Gdk.ModifierType
     let action = global.display.get_keybinding_action(keyCode, modifierState);
 
-    // Other bindings are only available to the user session when the overview is up and
-    // no modal dialog is present.
-    if (global.session_type == Shell.SessionType.USER && (!overview.visible || modalCount > 1))
-        return false;
-
     // This isn't a Meta.KeyBindingAction yet
     if (symbol == Clutter.Super_L || symbol == Clutter.Super_R) {
         overview.hide();
@@ -637,28 +587,39 @@ function _globalKeyPressHandler(actor, event) {
         return true;
     }
 
-    // None of the other bindings are relevant outside of the user's session
-    if (global.session_type != Shell.SessionType.USER)
-        return false;
-
     switch (action) {
         // left/right would effectively act as synonyms for up/down if we enabled them;
         // but that could be considered confusing; we also disable them in the main view.
         //
         // case Meta.KeyBindingAction.WORKSPACE_LEFT:
+        //  if (!sessionMode.hasWorkspaces)
+        //      return false;
+        //
         //     wm.actionMoveWorkspaceLeft();
         //     return true;
         // case Meta.KeyBindingAction.WORKSPACE_RIGHT:
+        //  if (!sessionMode.hasWorkspaces)
+        //      return false;
+        //
         //     wm.actionMoveWorkspaceRight();
         //     return true;
         case Meta.KeyBindingAction.WORKSPACE_UP:
+            if (!sessionMode.hasWorkspaces)
+                return false;
+
             wm.actionMoveWorkspaceUp();
             return true;
         case Meta.KeyBindingAction.WORKSPACE_DOWN:
+            if (!sessionMode.hasWorkspaces)
+                return false;
+
             wm.actionMoveWorkspaceDown();
             return true;
         case Meta.KeyBindingAction.PANEL_RUN_DIALOG:
         case Meta.KeyBindingAction.COMMAND_2:
+            if (!sessionMode.hasRunDialog)
+                return false;
+
             getRunDialog().open();
             return true;
         case Meta.KeyBindingAction.PANEL_MAIN_MENU:
@@ -931,7 +892,8 @@ function initializeDeferredWork(actor, callback, props) {
 function queueDeferredWork(workId) {
     let data = _deferredWorkData[workId];
     if (!data) {
-        global.logError('invalid work id ', workId);
+        let message = 'Invalid work id %d'.format(workId);
+        logError(new Error(message), message);
         return;
     }
     if (_deferredWorkQueue.indexOf(workId) < 0)
