@@ -483,7 +483,7 @@ const NotificationDaemon = new Lang.Class({
     },
 
     _onTrayIconAdded: function(o, icon) {
-        let source = this._getSource(icon.title || icon.wm_class || _("Unknown"), icon.pid, null, null, icon);
+        let source = this._getSource(icon.title || icon.wm_class || C_("program", "Unknown"), icon.pid, null, null, icon);
     },
 
     _onTrayIconRemoved: function(o, icon) {
@@ -578,11 +578,27 @@ const Source = new Lang.Class({
         return true;
     },
 
+    _getApp: function() {
+        let app;
+
+        app = Shell.WindowTracker.get_default().get_app_from_pid(this.pid);
+        if (app != null)
+            return app;
+
+        if (this.trayIcon) {
+            app = Shell.AppSystem.get_default().lookup_wmclass(this.trayIcon.wmclass);
+            if (app != null)
+                return app;
+        }
+
+        return null;
+    },
+
     _setApp: function() {
         if (this.app)
             return;
 
-        this.app = Shell.WindowTracker.get_default().get_app_from_pid(this.pid);
+        this.app = this._getApp();
         if (!this.app)
             return;
 
@@ -622,5 +638,10 @@ const Source = new Lang.Class({
         }
 
         this.parent();
+    },
+
+    createNotificationIcon: function() {
+        // We set the summary icon ourselves.
+        return null;
     }
 });
