@@ -14,7 +14,6 @@ const St = imports.gi.St;
 const Signals = imports.signals;
 const Atk = imports.gi.Atk;
 
-const Config = imports.misc.config;
 const CtrlAltTab = imports.ui.ctrlAltTab;
 const DND = imports.ui.dnd;
 const Layout = imports.ui.layout;
@@ -32,33 +31,6 @@ const BUTTON_DND_ACTIVATION_TIMEOUT = 250;
 const ANIMATED_ICON_UPDATE_TIMEOUT = 100;
 const SPINNER_ANIMATION_TIME = 0.2;
 
-const STANDARD_STATUS_AREA_ORDER = ['a11y', 'keyboard', 'volume', 'bluetooth', 'network', 'battery', 'userMenu'];
-const STANDARD_STATUS_AREA_SHELL_IMPLEMENTATION = {
-    'a11y': imports.ui.status.accessibility.ATIndicator,
-    'volume': imports.ui.status.volume.Indicator,
-    'battery': imports.ui.status.power.Indicator,
-    'keyboard': imports.ui.status.keyboard.XKBIndicator,
-    'userMenu': imports.ui.userMenu.UserMenuButton
-};
-
-if (Config.HAVE_BLUETOOTH)
-    STANDARD_STATUS_AREA_SHELL_IMPLEMENTATION['bluetooth'] = imports.ui.status.bluetooth.Indicator;
-
-try {
-    STANDARD_STATUS_AREA_SHELL_IMPLEMENTATION['network'] = imports.ui.status.network.NMApplet;
-} catch(e) {
-    log('NMApplet is not supported. It is possible that your NetworkManager version is too old');
-}
-
-const GDM_STATUS_AREA_ORDER = ['a11y', 'display', 'keyboard', 'volume', 'battery', 'powerMenu'];
-const GDM_STATUS_AREA_SHELL_IMPLEMENTATION = {
-    'a11y': imports.ui.status.accessibility.ATIndicator,
-    'volume': imports.ui.status.volume.Indicator,
-    'battery': imports.ui.status.power.Indicator,
-    'keyboard': imports.ui.status.keyboard.XKBIndicator,
-    'powerMenu': imports.gdm.powerMenu.PowerMenuButton
-};
-
 // To make sure the panel corners blend nicely with the panel,
 // we draw background and borders the same way, e.g. drawing
 // them as filled shapes from the outside inwards instead of
@@ -962,7 +934,7 @@ const Panel = new Lang.Class({
         this.actor.connect('button-press-event', Lang.bind(this, this._onButtonPress));
 
         /* Button on the left side of the panel. */
-        if (global.session_type == Shell.SessionType.USER) {
+        if (Main.sessionMode.hasOverview) {
             this._activitiesButton = new ActivitiesButton();
             this._activities = this._activitiesButton.actor;
             this._leftBox.add(this._activities);
@@ -970,28 +942,19 @@ const Panel = new Lang.Class({
             // The activities button has a pretend menu, so as to integrate
             // more cleanly with the rest of the panel
             this._menus.addMenu(this._activitiesButton.menu);
+        }
 
+        if (Main.sessionMode.hasAppMenu) {
             this._appMenu = new AppMenuButton(this._menus);
             this._leftBox.add(this._appMenu.actor);
         }
 
         /* center */
-        if (global.session_type == Shell.SessionType.USER)
-            this._dateMenu = new DateMenu.DateMenuButton({ showEvents: true });
-        else
-            this._dateMenu = new DateMenu.DateMenuButton({ showEvents: false });
+        this._dateMenu = new DateMenu.DateMenuButton();
         this._centerBox.add(this._dateMenu.actor, { y_fill: true });
         this._menus.addMenu(this._dateMenu.menu);
 
         /* right */
-        if (global.session_type == Shell.SessionType.GDM) {
-            this._status_area_order = GDM_STATUS_AREA_ORDER;
-            this._status_area_shell_implementation = GDM_STATUS_AREA_SHELL_IMPLEMENTATION;
-        } else {
-            this._status_area_order = STANDARD_STATUS_AREA_ORDER;
-            this._status_area_shell_implementation = STANDARD_STATUS_AREA_SHELL_IMPLEMENTATION;
-        }
-
         Main.statusIconDispatcher.connect('status-icon-added', Lang.bind(this, this._onTrayIconAdded));
         Main.statusIconDispatcher.connect('status-icon-removed', Lang.bind(this, this._onTrayIconRemoved));
 
@@ -1115,17 +1078,17 @@ const Panel = new Lang.Class({
 
     openAppMenu: function() {
         let menu = this._appMenu.menu;
-        if (Main.overview.visible || menu.isOpen)
-          return;
+        if (!this._appMenu.actor.reactive || menu.isOpen)
+            return;
 
         menu.open();
         menu.actor.navigate_focus(null, Gtk.DirectionType.TAB_FORWARD, false);
     },
 
     startStatusArea: function() {
-        for (let i = 0; i < this._status_area_order.length; i++) {
-            let role = this._status_area_order[i];
-            let constructor = this._status_area_shell_implementation[role];
+        for (let i = 0; i < Main.sessionMode.statusArea.order.length; i++) {
+            let role = Main.sessionMode.statusArea.order[i];
+            let constructor = Main.sessionMode.statusArea.implementation[role];
             if (!constructor) {
                 // This icon is not implemented (this is a bug)
                 continue;
@@ -1175,18 +1138,21 @@ const Panel = new Lang.Class({
     },
 
     _onTrayIconAdded: function(o, icon, role) {
-        if (this._status_area_shell_implementation[role]) {
+        if (Main.sessionMode.statusArea.implementation[role]) {
             // This icon is legacy, and replaced by a Shell version
             // Hide it
             return;
         }
 
+        if (Main.sessionMode.statusArea.order.indexOf(role) == -1)
+            return;
+
         icon.height = PANEL_ICON_SIZE;
         let buttonBox = new PanelMenu.ButtonBox();
         let box = buttonBox.actor;
         box.add_actor(icon);
 
-        this._insertStatusItem(box, this._status_area_order.indexOf(role));
+        this._insertStatusItem(box, Main.sessionMode.statusArea.order.indexOf(role));
     },
 
     _onTrayIconRemoved: function(o, icon) {
