@@ -546,6 +546,10 @@ const PopupSliderMenuItem = new Lang.Class({
         this._slider.connect('repaint', Lang.bind(this, this._sliderRepaint));
         this.actor.connect('button-press-event', Lang.bind(this, this._startDragging));
         this.actor.connect('scroll-event', Lang.bind(this, this._onScrollEvent));
+        this.actor.connect('notify::mapped', Lang.bind(this, function() {
+            if (!this.actor.mapped)
+                this._endDragging();
+        }));
 
         this._releaseId = this._motionId = 0;
         this._dragging = false;
@@ -884,8 +888,7 @@ const PopupMenuBase = new Lang.Class({
     },
 
     addSettingsAction: function(title, desktopFile) {
-        // Don't allow user settings to get edited unless we're in a user session
-        if (global.session_type != Shell.SessionType.USER)
+        if (!Main.sessionMode.allowSettings)
             return null;
 
         let menuItem = this.addAction(title, function() {
