@@ -18,9 +18,7 @@ const DISABLED_OPEN_SEARCH_PROVIDERS_KEY = 'disabled-open-search-providers';
 const MatchType = {
     NONE: 0,
     SUBSTRING: 1,
-    MULTIPLE_SUBSTRING: 2,
-    PREFIX: 3,
-    MULTIPLE_PREFIX: 4
+    PREFIX: 2
 };
 
 const SearchResultDisplay = new Lang.Class({
@@ -72,11 +70,8 @@ const SearchResultDisplay = new Lang.Class({
  * Subclass this object to add a new result type
  * to the search system, then call registerProvider()
  * in SearchSystem with an instance.
- * By default, search is synchronous and uses the
+ * Search is asynchronous and uses the
  * getInitialResultSet()/getSubsearchResultSet() methods.
- * For asynchronous search, set the async property to true
- * and implement getInitialResultSetAsync()/getSubsearchResultSetAsync()
- * instead.
  */
 const SearchProvider = new Lang.Class({
     Name: 'SearchProvider',
@@ -84,7 +79,6 @@ const SearchProvider = new Lang.Class({
     _init: function(title) {
         this.title = title;
         this.searchSystem = null;
-        this.async = false;
     },
 
     /**
@@ -95,7 +89,7 @@ const SearchProvider = new Lang.Class({
      * therefore a single term of length one or two), or when
      * a new term is added.
      *
-     * Should return an array of result identifier strings representing
+     * Should "return" an array of result identifier strings representing
      * items which match the given search terms.  This
      * is expected to be a substring match on the metadata for a given
      * item.  Ordering of returned results is up to the discretion of the provider,
@@ -105,6 +99,9 @@ const SearchProvider = new Lang.Class({
      *    description) before single matches
      *  * Put items which match on a prefix before non-prefix substring matches
      *
+     * We say "return" above, but in order to make the query asynchronous, use
+     * this.searchSystem.pushResults();. The return value should be ignored.
+     *
      * This function should be fast; do not perform unindexed full-text searches
      * or network queries.
      */
@@ -112,18 +109,6 @@ const SearchProvider = new Lang.Class({
         throw new Error('Not implemented');
     },
 
-    /**
-     * getInitialResultSetAsync:
-     * @terms: Array of search terms, treated as logical AND
-     *
-     * Like getInitialResultSet(), but the method should return immediately
-     * without a return value - use SearchSystem.pushResults() when the
-     * corresponding results are ready.
-     */
-    getInitialResultSetAsync: function(terms) {
-        throw new Error('Not implemented');
-    },
-
     /**
      * getSubsearchResultSet:
      * @previousResults: Array of item identifiers
@@ -136,46 +121,23 @@ const SearchProvider = new Lang.Class({
      *
      * This allows search providers to only search through the previous
      * result set, rather than possibly performing a full re-query.
+     *
+     * Similar to getInitialResultSet, the return value for this will
+     * be ignored; use this.searchSystem.pushResults();.
      */
     getSubsearchResultSet: function(previousResults, newTerms) {
         throw new Error('Not implemented');
     },
 
-    /**
-     * getSubsearchResultSetAsync:
-     * @previousResults: Array of item identifiers
-     * @newTerms: Updated search terms
-     *
-     * Like getSubsearchResultSet(), but the method should return immediately
-     * without a return value - use SearchSystem.pushResults() when the
-     * corresponding results are ready.
-     */
-    getSubsearchResultSetAsync: function(previousResults, newTerms) {
-        throw new Error('Not implemented');
-    },
-
     /**
      * getResultMetas:
      * @ids: Result identifier strings
      *
-     * Return an array of objects with 'id', 'name', (both strings) and
+     * Call callback with array of objects with 'id', 'name', (both strings) and
      * 'createIcon' (function(size) returning a Clutter.Texture) properties
      * with the same number of members as @ids
      */
-    getResultMetas: function(ids) {
-        throw new Error('Not implemented');
-    },
-
-    /**
-     * getResultMetasAsync:
-     * @ids: Result identifier strings
-     * @callback: callback to pass the results to when ready
-     *
-     * Like getResultMetas(), but the method should return immediately
-     * without a return value - pass the results to the provided @callback
-     * when ready.
-     */
-    getResultMetasAsync: function(ids, callback) {
+    getResultMetas: function(ids, callback) {
         throw new Error('Not implemented');
     },
 
@@ -379,42 +341,33 @@ const SearchSystem = new Lang.Class({
             }
         }
 
+        let previousResultsArr = this._previousResults;
+
         let results = [];
+        this._previousTerms = terms;
+        this._previousResults = results;
+
         if (isSubSearch) {
             for (let i = 0; i < this._providers.length; i++) {
-                let [provider, previousResults] = this._previousResults[i];
+                let [provider, previousResults] = previousResultsArr[i];
                 try {
-                    if (provider.async) {
-                        provider.getSubsearchResultSetAsync(previousResults, terms);
-                        results.push([provider, []]);
-                    } else {
-                        let providerResults = provider.getSubsearchResultSet(previousResults, terms);
-                        results.push([provider, providerResults]);
-                    }
+                    results.push([provider, []]);
+                    provider.getSubsearchResultSet(previousResults, terms);
                 } catch (error) {
-                    global.log ('A ' + error.name + ' has occured in ' + provider.title + ': ' + error.message);
+                    log('A ' + error.name + ' has occured in ' + provider.title + ': ' + error.message);
                 }
             }
         } else {
             for (let i = 0; i < this._providers.length; i++) {
                 let provider = this._providers[i];
                 try {
-                    if (provider.async) {
-                        provider.getInitialResultSetAsync(terms);
-                        results.push([provider, []]);
-                    } else {
-                        let providerResults = provider.getInitialResultSet(terms);
-                        results.push([provider, providerResults]);
-                    }
+                    results.push([provider, []]);
+                    provider.getInitialResultSet(terms);
                 } catch (error) {
-                    global.log ('A ' + error.name + ' has occured in ' + provider.title + ': ' + error.message);
+                    log('A ' + error.name + ' has occured in ' + provider.title + ': ' + error.message);
                 }
             }
         }
-
-        this._previousTerms = terms;
-        this._previousResults = results;
-        this.emit('search-completed', results);
     },
 });
 Signals.addSignalMethods(SearchSystem.prototype);
