@@ -5,6 +5,7 @@ const Lang = imports.lang;
 const Gtk = imports.gi.Gtk;
 const Meta = imports.gi.Meta;
 const St = imports.gi.St;
+const Atk = imports.gi.Atk;
 
 const DND = imports.ui.dnd;
 const IconGrid = imports.ui.iconGrid;
@@ -33,12 +34,13 @@ const SearchResult = new Lang.Class({
             content = new St.Bin({ style_class: 'search-result-content',
                                    reactive: true,
                                    can_focus: true,
-                                   track_hover: true });
+                                   track_hover: true,
+                                   accessible_role: Atk.Role.PUSH_BUTTON });
             let icon = new IconGrid.BaseIcon(this.metaInfo['name'],
                                              { createIcon: this.metaInfo['createIcon'] });
             content.set_child(icon.actor);
             this._dragActorSource = icon.icon;
-            this.actor.label_actor = icon.label;
+            content.label_actor = icon.label;
         } else {
             if (content._delegate && content._delegate.getDragActorSource)
                 this._dragActorSource = content._delegate.getDragActorSource();
@@ -119,13 +121,7 @@ const GridSearchResults = new Lang.Class({
                 if (results.length == 0)
                     return;
 
-                if (provider.async) {
-                    provider.getResultMetasAsync(results,
-                                                 Lang.bind(this, this.renderResults));
-                } else {
-                    let metas = provider.getResultMetas(results);
-                    this.renderResults(metas);
-                }
+                provider.getResultMetas(results, Lang.bind(this, this.renderResults));
             }));
         }));
         this._notDisplayedResult = [];
@@ -135,7 +131,7 @@ const GridSearchResults = new Lang.Class({
 
     getResultsForDisplay: function() {
         let alreadyVisible = this._pendingClear ? 0 : this._grid.visibleItemsCount();
-        let canDisplay = this._grid.childrenInRow(this._width) * MAX_SEARCH_RESULTS_ROWS
+        let canDisplay = this._grid.childrenInRow(this._width) * this._grid.getRowLimit()
                          - alreadyVisible;
 
         let numResults = Math.min(this._notDisplayedResult.length, canDisplay);
@@ -179,8 +175,7 @@ const SearchResults = new Lang.Class({
 
     _init: function(searchSystem, openSearchSystem) {
         this._searchSystem = searchSystem;
-        this._searchSystem.connect('search-updated', Lang.bind(this, this._updateCurrentResults));
-        this._searchSystem.connect('search-completed', Lang.bind(this, this._updateResults));
+        this._searchSystem.connect('search-updated', Lang.bind(this, this._updateResults));
         this._openSearchSystem = openSearchSystem;
 
         this.actor = new St.BoxLayout({ name: 'searchResults',
@@ -214,10 +209,8 @@ const SearchResults = new Lang.Class({
         this._content.add(this._statusText);
         this._providers = this._searchSystem.getProviders();
         this._providerMeta = [];
-        this._providerMetaResults = {};
         for (let i = 0; i < this._providers.length; i++) {
             this.createProviderMeta(this._providers[i]);
-            this._providerMetaResults[this.providers[i].title] = [];
         }
         this._searchProvidersBox = new St.BoxLayout({ style_class: 'search-providers-box' });
         this.actor.add(this._searchProvidersBox);
@@ -290,8 +283,7 @@ const SearchResults = new Lang.Class({
 
         this._providerMeta.push({ provider: provider,
                                   actor: providerBox,
-                                  resultDisplay: resultDisplay,
-                                  hasPendingResults: false });
+                                  resultDisplay: resultDisplay });
         this._content.add(providerBox);
     },
 
@@ -307,7 +299,6 @@ const SearchResults = new Lang.Class({
     },
 
     _clearDisplay: function() {
-        this._visibleResultsCount = 0;
         for (let i = 0; i < this._providerMeta.length; i++) {
             let meta = this._providerMeta[i];
             meta.resultDisplay.clear();
@@ -335,6 +326,8 @@ const SearchResults = new Lang.Class({
 
     doSearch: function (searchString) {
         this._searchSystem.updateSearch(searchString);
+        let terms = this._searchSystem.getTerms();
+        this._openSearchSystem.setSearchTerms(terms);
     },
 
     _metaForProvider: function(provider) {
@@ -346,8 +339,6 @@ const SearchResults = new Lang.Class({
 
         for (let i = 0; i < this._providerMeta.length; i++) {
             let meta = this._providerMeta[i];
-            if (meta.hasPendingResults)
-                return;
 
             if (!meta.actor.visible)
                 continue;
@@ -372,71 +363,57 @@ const SearchResults = new Lang.Class({
         }
     },
 
-    _updateCurrentResults: function(searchSystem, results) {
-        let terms = searchSystem.getTerms();
-        let [provider, providerResults] = results;
-        let meta = this._metaForProvider(provider);
-        meta.hasPendingResults = false;
-        this._updateProviderResults(provider, providerResults, terms);
-    },
+    _updateStatusText: function () {
+        let haveResults = false;
 
-    _updateProviderResults: function(provider, providerResults, terms) {
-        let meta = this._metaForProvider(provider);
-        if (providerResults.length == 0) {
-            this._clearDisplayForProvider(provider);
-            meta.resultDisplay.setResults([], []);
-        } else {
-            this._providerMetaResults[provider.title] = providerResults;
-            meta.resultDisplay.setResults(providerResults, terms);
-            let results = meta.resultDisplay.getResultsForDisplay();
-
-            if (provider.async) {
-                provider.getResultMetasAsync(results, Lang.bind(this,
-                    function(metas) {
-                        this._clearDisplayForProvider(provider);
-                        meta.actor.show();
-                        this._content.hide();
-                        meta.resultDisplay.renderResults(metas);
-                        this._maybeSetInitialSelection();
-                        this._content.show();
-                    }));
-            } else {
-                let metas = provider.getResultMetas(results);
-                this._clearDisplayForProvider(provider);
-                meta.actor.show();
-                meta.resultDisplay.renderResults(metas);
+        for (let i = 0; i < this._providerMeta.length; ++i)
+            if (this._providerMeta[i].resultDisplay.getFirstResult()) {
+                haveResults = true;
+                break;
             }
-        }
-        this._maybeSetInitialSelection();
-    },
 
-    _updateResults: function(searchSystem, results) {
-        if (results.length == 0) {
+        if (!haveResults) {
             this._statusText.set_text(_("No matching results."));
             this._statusText.show();
         } else {
             this._statusText.hide();
         }
+    },
 
+    _updateResults: function(searchSystem, results) {
         let terms = searchSystem.getTerms();
-        this._openSearchSystem.setSearchTerms(terms);
+        let [provider, providerResults] = results;
+        let meta = this._metaForProvider(provider);
 
-        // To avoid CSS transitions causing flickering when the first search
-        // result stays the same, we hide the content while filling in the
-        // results.
-        this._content.hide();
+        if (providerResults.length == 0) {
+            this._clearDisplayForProvider(provider);
+            meta.resultDisplay.setResults([], []);
+            this._maybeSetInitialSelection();
+            this._updateStatusText();
+        } else {
+            meta.resultDisplay.setResults(providerResults, terms);
+            let results = meta.resultDisplay.getResultsForDisplay();
 
-        for (let i = 0; i < results.length; i++) {
-            let [provider, providerResults] = results[i];
-            let meta = this._metaForProvider(provider);
-            meta.hasPendingResults = provider.async;
-            if (!meta.hasPendingResults)
-                this._updateProviderResults(provider, providerResults, terms);
+            provider.getResultMetas(results, Lang.bind(this, function(metas) {
+                this._clearDisplayForProvider(provider);
+                meta.actor.show();
+
+                // Hiding drops the key focus if we have it
+                let focus = global.stage.get_key_focus();
+                // To avoid CSS transitions causing flickering when
+                // the first search result stays the same, we hide the
+                // content while filling in the results.
+                this._content.hide();
+
+                meta.resultDisplay.renderResults(metas);
+                this._maybeSetInitialSelection();
+                this._updateStatusText();
+
+                this._content.show();
+                if (this._content.contains(focus))
+                    global.stage.set_key_focus(focus);
+            }));
         }
-
-        this._content.show();
-
-        return true;
     },
 
     activateDefault: function() {
