@@ -48,11 +48,6 @@ function _setLabelsForMessage(dialog, message) {
         _setLabelText(dialog.descriptionLabel, labels[1]);
 }
 
-function _createIcon(gicon) {
-    return new St.Icon({ gicon: gicon,
-                         style_class: 'shell-mount-operation-icon' })
-}
-
 /* -------------------------------------------------------- */
 
 const ListItem = new Lang.Class({
@@ -114,11 +109,12 @@ const ShellMountOperation = new Lang.Class({
         this.mountOp.connect('aborted',
                              Lang.bind(this, this._onAborted));
 
-        this._gicon = source.get_icon();
+        this._icon = new St.Icon({ gicon: source.get_icon(),
+                                   style_class: 'shell-mount-operation-icon' });
     },
 
     _onAskQuestion: function(op, message, choices) {
-        this._dialog = new ShellMountQuestionDialog(this._gicon);
+        this._dialog = new ShellMountQuestionDialog(this._icon);
 
         this._dialog.connect('response',
                                Lang.bind(this, function(object, choice) {
@@ -135,7 +131,7 @@ const ShellMountOperation = new Lang.Class({
 
     _onAskPassword: function(op, message) {
         this._notificationShowing = true;
-        this._source = new ShellMountPasswordSource(message, this._gicon, this._reaskPassword);
+        this._source = new ShellMountPasswordSource(message, this._icon, this._reaskPassword);
 
         this._source.connect('password-ready',
                              Lang.bind(this, function(source, password) {
@@ -170,7 +166,7 @@ const ShellMountOperation = new Lang.Class({
         let message = op.get_show_processes_message();
 
         if (!this._processesDialog) {
-            this._processesDialog = new ShellProcessesDialog(this._gicon);
+            this._processesDialog = new ShellProcessesDialog(this._icon);
             this._dialog = this._processesDialog;
 
             this._processesDialog.connect('response', 
@@ -196,14 +192,14 @@ const ShellMountQuestionDialog = new Lang.Class({
     Name: 'ShellMountQuestionDialog',
     Extends: ModalDialog.ModalDialog,
 
-    _init: function(gicon) {
+    _init: function(icon) {
         this.parent({ styleClass: 'mount-question-dialog' });
 
         let mainContentLayout = new St.BoxLayout();
         this.contentLayout.add(mainContentLayout, { x_fill: true,
                                                     y_fill: false });
 
-        this._iconBin = new St.Bin({ child: _createIcon(gicon) });
+        this._iconBin = new St.Bin({ child: icon });
         mainContentLayout.add(this._iconBin,
                               { x_fill:  true,
                                 y_fill:  false,
@@ -242,21 +238,17 @@ const ShellMountPasswordSource = new Lang.Class({
     Name: 'ShellMountPasswordSource',
     Extends: MessageTray.Source,
 
-    _init: function(message, gicon, reaskPassword) {
-        this._gicon = gicon;
-
+    _init: function(message, icon, reaskPassword) {
         let strings = message.split('\n');
         this.parent(strings[0]);
-        this._notification = new ShellMountPasswordNotification(this, strings, reaskPassword);
+
+        this._notification = new ShellMountPasswordNotification(this, strings, icon, reaskPassword);
+        this._setSummaryIcon(icon);
 
         // add ourselves as a source, and popup the notification
         Main.messageTray.add(this);
         this.notify(this._notification);
     },
-
-    createNotificationIcon: function() {
-        return _createIcon(this._gicon);
-    },
 });
 Signals.addSignalMethods(ShellMountPasswordSource.prototype);
 
@@ -264,8 +256,8 @@ const ShellMountPasswordNotification = new Lang.Class({
     Name: 'ShellMountPasswordNotification',
     Extends: MessageTray.Notification,
 
-    _init: function(source, strings, reaskPassword) {
-        this.parent(source, strings[0], null, { customContent: true });
+    _init: function(source, strings, icon, reaskPassword) {
+        this.parent(source, strings[0], null, { customContent: true, icon: icon });
 
         // set the notification to transient and urgent, so that it
         // expands out
@@ -306,14 +298,14 @@ const ShellProcessesDialog = new Lang.Class({
     Name: 'ShellProcessesDialog',
     Extends: ModalDialog.ModalDialog,
 
-    _init: function(gicon) {
+    _init: function(icon) {
         this.parent({ styleClass: 'show-processes-dialog' });
 
         let mainContentLayout = new St.BoxLayout();
         this.contentLayout.add(mainContentLayout, { x_fill: true,
                                                     y_fill: false });
 
-        this._iconBin = new St.Bin({ child: _createIcon(gicon) });
+        this._iconBin = new St.Bin({ child: icon });
         mainContentLayout.add(this._iconBin,
                               { x_fill:  true,
                                 y_fill:  false,
