@@ -304,9 +304,10 @@ const NMDevice = new Lang.Class({
             // record the connection
             let obj = {
                 connection: connections[i],
-                name: connections[i]._name,
-                uuid: connections[i]._uuid,
+                name: connections[i].get_id(),
+                uuid: connections[i].get_uuid(),
                 timestamp: connections[i]._timestamp,
+                item: null,
             };
             this._connections.push(obj);
         }
@@ -401,48 +402,46 @@ const NMDevice = new Lang.Class({
     },
 
     checkConnection: function(connection) {
-        let pos = this._findConnection(connection._uuid);
+        let pos = this._findConnection(connection.get_uuid());
         let exists = pos != -1;
         let valid = this.connectionValid(connection);
+        let similar = false;
+        if (exists) {
+            let existing = this._connections[pos];
 
-        if (exists && !valid)
-            this.removeConnection(connection);
-        else if (!exists && valid)
-            this.addConnection(connection);
-        else if (exists && valid) {
-            // propagate changes and update the UI
-
-            if (this._connections[pos].timestamp != connection._timestamp) {
-                this._connections[pos].timestamp = connection._timestamp;
-                this._connections.sort(this._connectionSortFunction);
-
-                this._clearSection();
-                this._queueCreateSection();
-            }
+            // Check if connection changed name or id
+            similar = existing.name == connection.get_id() &&
+                existing.timestamp == connection._timestamp;
         }
+
+        if (exists && valid && similar) {
+            // Nothing to do
+            return;
+        }
+
+        if (exists)
+            this.removeConnection(connection);
+        if (valid)
+            this.addConnection(connection);
     },
 
     addConnection: function(connection) {
         // record the connection
         let obj = {
             connection: connection,
-            name: connection._name,
-            uuid: connection._uuid,
+            name: connection.get_id(),
+            uuid: connection.get_uuid(),
             timestamp: connection._timestamp,
+            item: null,
         };
-        this._connections.push(obj);
-        this._connections.sort(this._connectionSortFunction);
+        Util.insertSorted(this._connections, obj, this._connectionSortFunction);
 
         this._clearSection();
         this._queueCreateSection();
     },
 
     removeConnection: function(connection) {
-        if (!connection._uuid) {
-            log('Cannot remove a connection without an UUID');
-            return;
-        }
-        let pos = this._findConnection(connection._uuid);
+        let pos = this._findConnection(connection.get_uuid());
         if (pos == -1) {
             // this connection was never added, nothing to do here
             return;
@@ -707,18 +706,15 @@ const NMDeviceWired = new Lang.Class({
         // the device
         // we can do it here because addConnection and removeConnection
         // both call _createSection at some point
-        if (this._connections.length <= 1)
-            this.section.actor.hide();
-        else
-            this.section.actor.show();
+        this.section.actor.visible = this._connections.length > 1;
     },
 
     _createAutomaticConnection: function() {
         let connection = new NetworkManager.Connection();
-        connection._uuid = NetworkManager.utils_uuid_generate();
+        let uuid = NetworkManager.utils_uuid_generate();
         connection.add_setting(new NetworkManager.SettingWired());
         connection.add_setting(new NetworkManager.SettingConnection({
-            uuid: connection._uuid,
+            uuid: uuid,
             id: this._autoConnectionName,
             type: NetworkManager.SETTING_WIRED_SETTING_NAME,
             autoconnect: true
@@ -862,10 +858,10 @@ const NMDeviceBluetooth = new Lang.Class({
 
     _createAutomaticConnection: function() {
         let connection = new NetworkManager.Connection;
-        connection._uuid = NetworkManager.utils_uuid_generate();
+        let uuid = NetworkManager.utils_uuid_generate();
         connection.add_setting(new NetworkManager.SettingBluetooth);
         connection.add_setting(new NetworkManager.SettingConnection({
-            uuid: connection._uuid,
+            uuid: uuid,
             id: this._autoConnectionName,
             type: NetworkManager.SETTING_BLUETOOTH_SETTING_NAME,
             autoconnect: false
@@ -900,12 +896,12 @@ const NMDeviceVPN = new Lang.Class({
     Name: 'NMDeviceVPN',
     Extends: NMDevice,
 
-    _init: function(client) {
+    _init: function(client, device, connections) {
         // Disable autoconnections
         this._autoConnectionName = null;
         this.category = NMConnectionCategory.VPN;
 
-        this.parent(client, null, [ ]);
+        this.parent(client, null, connections);
     },
 
     connectionValid: function(connection) {
@@ -917,13 +913,24 @@ const NMDeviceVPN = new Lang.Class({
     },
 
     get connected() {
-        return !!this._activeConnection;
+        if (!this._activeConnection)
+            return false;
+
+        return this._activeConnection.vpn_state == NetworkManager.VPNConnectionState.ACTIVATED;
     },
 
     setActiveConnection: function(activeConnection) {
+        if (this._stateChangeId)
+            this._activeConnection.disconnect(this._stateChangeId);
+        this._stateChangeId = 0;
+
         this.parent(activeConnection);
 
-        this.emit('active-connection-changed');
+        if (this._activeConnection)
+            this._stateChangeId = this._activeConnection.connect('vpn-state-changed',
+                                                                 Lang.bind(this, this._connectionStateChanged));
+
+        this.emit('state-changed');
     },
 
     _shouldShowConnectionList: function() {
@@ -936,7 +943,39 @@ const NMDeviceVPN = new Lang.Class({
     },
 
     getStatusLabel: function() {
-        return null;
+        if (!this._activeConnection) // Same as DISCONNECTED
+            return null;
+
+        switch(this._activeConnection.vpn_state) {
+        case NetworkManager.VPNConnectionState.DISCONNECTED:
+        case NetworkManager.VPNConnectionState.ACTIVATED:
+            return null;
+        case NetworkManager.VPNConnectionState.PREPARE:
+        case NetworkManager.VPNConnectionState.CONNECT:
+        case NetworkManager.VPNConnectionState.IP_CONFIG_GET:
+            return _("connecting...");
+        case NetworkManager.VPNConnectionState.NEED_AUTH:
+            /* Translators: this is for network connections that require some kind of key or password */
+            return _("authentication required");
+        case NetworkManager.VPNConnectionState.FAILED:
+            return _("connection failed");
+        default:
+            log('VPN connection state invalid, is %d'.format(this.device.state));
+            return 'invalid';
+        }
+    },
+
+    _connectionStateChanged: function(connection, newstate, reason) {
+        if (newstate == NetworkManager.VPNConnectionState.FAILED) {
+            // FIXME: if we ever want to show something based on reason,
+            // we need to convert from NetworkManager.VPNConnectionStateReason
+            // to NetworkManager.DeviceStateReason
+            this.emit('activation-failed', reason);
+        }
+
+        // Differently from real NMDevices, there is no need to queue
+        // an update of the menu section, contents wouldn't change anyway
+        this.emit('state-changed');
     }
 });
 
@@ -1038,13 +1077,8 @@ const NMDeviceWireless = new Lang.Class({
     },
 
     setEnabled: function(enabled) {
-        if (enabled) {
-            this.statusItem.actor.show();
-            this.section.actor.show();
-        } else {
-            this.statusItem.actor.hide();
-            this.section.actor.hide();
-        }
+        this.statusItem.actor.visible = enabled;
+        this.section.actor.visible = enabled;
     },
 
     activate: function() {
@@ -1331,9 +1365,7 @@ const NMDeviceWireless = new Lang.Class({
     },
 
     removeConnection: function(connection) {
-        if (!connection._uuid)
-            return;
-        let pos = this._findConnection(connection._uuid);
+        let pos = this._findConnection(connection.get_uuid());
         if (pos == -1) {
             // removing connection that was never added
             return;
@@ -1347,7 +1379,7 @@ const NMDeviceWireless = new Lang.Class({
             let apObj = this._networks[i];
             let connections = apObj.connections;
             for (let k = 0; k < connections.length; k++) {
-                if (connections[k]._uuid == connection._uuid) {
+                if (connections[k].get_uuid() == connection.get_uuid()) {
                     // remove the connection from the access point group
                     connections.splice(k);
                     forceupdate = forceupdate || connections.length == 0;
@@ -1363,7 +1395,7 @@ const NMDeviceWireless = new Lang.Class({
                                 forceupdate = true;
                             } else {
                                 for (let j = 0; j < items.length; j++) {
-                                    if (items[j]._connection._uuid == connection._uuid) {
+                                    if (items[j]._connection.get_uuid() == connection.get_uuid()) {
                                         items[j].destroy();
                                         break;
                                     }
@@ -1390,8 +1422,8 @@ const NMDeviceWireless = new Lang.Class({
         // record the connection
         let obj = {
             connection: connection,
-            name: connection._name,
-            uuid: connection._uuid,
+            name: connection.get_id(),
+            uuid: connection.get_uuid(),
         };
         this._connections.push(obj);
 
@@ -1538,7 +1570,7 @@ const NMApplet = new Lang.Class({
     Extends: PanelMenu.SystemStatusButton,
 
     _init: function() {
-        this.parent('network-error', _("Network"));
+        this.parent('network-offline', _("Network"));
 
         this._client = NMClient.Client.new();
 
@@ -1552,6 +1584,15 @@ const NMApplet = new Lang.Class({
         this.menu.addMenuItem(this._statusSection);
         this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
 
+        this._activeConnections = [ ];
+        this._connections = [ ];
+
+        this._mainConnection = null;
+        this._activeAccessPointUpdateId = 0;
+        this._activeAccessPoint = null;
+        this._mobileUpdateId = 0;
+        this._mobileUpdateDevice = null;
+
         this._devices = { };
 
         this._devices.wired = {
@@ -1587,13 +1628,9 @@ const NMApplet = new Lang.Class({
 
         this._devices.vpn = {
             section: new PopupMenu.PopupMenuSection(),
-            device: new NMDeviceVPN(this._client),
+            device: this._makeWrapperDevice(NMDeviceVPN, null),
             item: new NMWiredSectionTitleMenuItem(_("VPN Connections"))
         };
-        this._devices.vpn.device.connect('active-connection-changed', Lang.bind(this, function() {
-            this._devices.vpn.item.updateForDevice(this._devices.vpn.device);
-        }));
-        this._devices.vpn.item.updateForDevice(this._devices.vpn.device);
         this._devices.vpn.section.addMenuItem(this._devices.vpn.item);
         this._devices.vpn.section.addMenuItem(this._devices.vpn.device.section);
         this._devices.vpn.section.actor.hide();
@@ -1601,15 +1638,6 @@ const NMApplet = new Lang.Class({
         this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
         this.menu.addSettingsAction(_("Network Settings"), 'gnome-network-panel.desktop');
 
-        this._activeConnections = [ ];
-        this._connections = [ ];
-
-        this._mainConnection = null;
-        this._activeAccessPointUpdateId = 0;
-        this._activeAccessPoint = null;
-        this._mobileUpdateId = 0;
-        this._mobileUpdateDevice = null;
-
         // Device types
         this._dtypes = { };
         this._dtypes[NetworkManager.DeviceType.ETHERNET] = NMDeviceWired;
@@ -1652,7 +1680,10 @@ const NMApplet = new Lang.Class({
 
     _ensureSource: function() {
         if (!this._source) {
-            this._source = new NMMessageTraySource();
+            this._source = new MessageTray.Source(_("Network Manager"),
+                                                  'network-transmit-receive',
+                                                  St.IconType.SYMBOLIC);
+
             this._source.connect('destroy', Lang.bind(this, function() {
                 this._source = null;
             }));
@@ -1673,6 +1704,18 @@ const NMApplet = new Lang.Class({
     },
 
     _syncSectionTitle: function(category) {
+        if (category == NMConnectionCategory.VPN) {
+            // Special case VPN: it's only one device (and a fake one
+            // actually), and we don't show it if empty
+            let device = this._devices.vpn.device;
+            let section = this._devices.vpn.section;
+            let item = this._devices.vpn.item;
+
+            section.actor.visible = !device.empty;
+            item.updateForDevice(device);
+            return;
+        }
+
         let devices = this._devices[category].devices;
         let item = this._devices[category].item;
         let section = this._devices[category].section;
@@ -1723,6 +1766,29 @@ const NMApplet = new Lang.Class({
         this._source.notify(device._notification);
     },
 
+    _makeWrapperDevice: function(wrapperClass, device) {
+        let wrapper = new wrapperClass(this._client, device, this._connections);
+
+        wrapper._activationFailedId = wrapper.connect('activation-failed', Lang.bind(this, function(device, reason) {
+            // XXX: nm-applet has no special text depending on reason
+            // but I'm not sure of this generic message
+            this._notifyForDevice(device, 'network-error',
+                                  _("Connection failed"),
+                                  _("Activation of network connection failed"),
+                                  MessageTray.Urgency.HIGH);
+        }));
+        wrapper._deviceStateChangedId = wrapper.connect('state-changed', Lang.bind(this, function(dev) {
+            this._syncSectionTitle(dev.category);
+        }));
+        wrapper._destroyId = wrapper.connect('destroy', function(wrapper) {
+            wrapper.disconnect(wrapper._activationFailedId);
+            wrapper.disconnect(wrapper._deviceStateChangedId);
+            wrapper.disconnect(wrapper._destroyId);
+        });
+
+        return wrapper;
+    },
+
     _deviceAdded: function(client, device) {
         if (device._delegate) {
             // already seen, not adding again
@@ -1730,24 +1796,8 @@ const NMApplet = new Lang.Class({
         }
         let wrapperClass = this._dtypes[device.get_device_type()];
         if (wrapperClass) {
-            let wrapper = new wrapperClass(this._client, device, this._connections);
+            let wrapper = this._makeWrapperDevice(wrapperClass, device);
 
-            wrapper._activationFailedId = wrapper.connect('activation-failed', Lang.bind(this, function(device, reason) {
-                // XXX: nm-applet has no special text depending on reason
-                // but I'm not sure of this generic message
-                this._notifyForDevice(device, 'network-error',
-                                      _("Connection failed"),
-                                      _("Activation of network connection failed"),
-                                     MessageTray.Urgency.HIGH);
-            }));
-            wrapper._deviceStateChangedId = wrapper.connect('state-changed', Lang.bind(this, function(dev) {
-                this._syncSectionTitle(dev.category);
-            }));
-            wrapper._destroyId = wrapper.connect('destroy', function(wrapper) {
-                wrapper.disconnect(wrapper._activationFailedId);
-                wrapper.disconnect(wrapper._deviceStateChangedId);
-                wrapper.disconnect(wrapper._destroyId);
-            });
             let section = this._devices[wrapper.category].section;
             let devices = this._devices[wrapper.category].devices;
 
@@ -1883,7 +1933,7 @@ const NMApplet = new Lang.Class({
         let connections = this._settings.list_connections();
         for (let i = 0; i < connections.length; i++) {
             let connection = connections[i];
-            if (connection._uuid) {
+            if (connection._updatedId) {
                 // connection was already seen (for example because NetworkManager was restarted)
                 continue;
             }
@@ -1896,7 +1946,7 @@ const NMApplet = new Lang.Class({
     },
 
     _newConnection: function(settings, connection) {
-        if (connection._uuid) {
+        if (connection._updatedId) {
             // connection was already seen
             return;
         }
@@ -1919,35 +1969,31 @@ const NMApplet = new Lang.Class({
 
         if (section == NMConnectionCategory.VPN) {
             this._devices.vpn.device.removeConnection(connection);
-            if (this._devices.vpn.device.empty)
-                this._devices.vpn.section.actor.hide();
+            this._syncSectionTitle(section);
         } else if (section != NMConnectionCategory.INVALID) {
             let devices = this._devices[section].devices;
             for (let i = 0; i < devices.length; i++)
                 devices[i].removeConnection(connection);
         }
 
-        connection._uuid = null;
         connection.disconnect(connection._removedId);
         connection.disconnect(connection._updatedId);
+        connection._removedId = connection._updatedId = 0;
     },
 
     _updateConnection: function(connection) {
         let connectionSettings = connection.get_setting_by_name(NetworkManager.SETTING_CONNECTION_SETTING_NAME);
         connection._type = connectionSettings.type;
-
         connection._section = this._ctypes[connection._type] || NMConnectionCategory.INVALID;
-        connection._name = connectionSettings.id;
-        connection._uuid = connectionSettings.uuid;
         connection._timestamp = connectionSettings.timestamp;
 
         let section = connection._section;
 
-        if (connection._section == NMConnectionCategory.INVALID)
+        if (section == NMConnectionCategory.INVALID)
             return;
         if (section == NMConnectionCategory.VPN) {
             this._devices.vpn.device.checkConnection(connection);
-            this._devices.vpn.section.actor.show();
+            this._syncSectionTitle(section);
         } else {
             let devices = this._devices[section].devices;
             for (let i = 0; i < devices.length; i++) {
@@ -1970,12 +2016,10 @@ const NMApplet = new Lang.Class({
 
         this._statusSection.actor.hide();
 
-        this._syncSectionTitle('wired');
-        this._syncSectionTitle('wireless');
-        this._syncSectionTitle('wwan');
-
-        if (!this._devices.vpn.device.empty)
-            this._devices.vpn.section.actor.show();
+        this._syncSectionTitle(NMConnectionCategory.WIRED);
+        this._syncSectionTitle(NMConnectionCategory.WIRELESS);
+        this._syncSectionTitle(NMConnectionCategory.WWAN);
+        this._syncSectionTitle(NMConnectionCategory.VPN);
     },
 
     _syncNMState: function() {
@@ -2108,18 +2152,3 @@ const NMApplet = new Lang.Class({
         }
     }
 });
-
-const NMMessageTraySource = new Lang.Class({
-    Name: 'NMMessageTraySource',
-    Extends: MessageTray.Source,
-
-    _init: function() {
-        this.parent(_("Network Manager"));
-
-        let icon = new St.Icon({ icon_name: 'network-transmit-receive',
-                                 icon_type: St.IconType.SYMBOLIC,
-                                 icon_size: this.ICON_SIZE
-                               });
-        this._setSummaryIcon(icon);
-    }
-});
