@@ -132,9 +132,6 @@ const Client = new Lang.Class({
             let channel = channels[i];
             let [targetHandle, targetHandleType] = channel.get_handle();
 
-            if (Shell.is_channel_invalidated(channel))
-              continue;
-
             /* Only observe contact text channels */
             if ((!(channel instanceof Tp.TextChannel)) ||
                targetHandleType != Tp.HandleType.CONTACT)
@@ -184,9 +181,6 @@ const Client = new Lang.Class({
                 continue;
             }
 
-            if (Shell.is_channel_invalidated(channel))
-              continue;
-
             // 'notify' will be true when coming from an actual HandleChannels
             // call, and not when from a successful Claim call. The point is
             // we don't want to notify for a channel we just claimed which
@@ -237,19 +231,12 @@ const Client = new Lang.Class({
         let channel = channels[0];
         let chanType = channel.get_channel_type();
 
-        if (Shell.is_channel_invalidated(channel)) {
-            Shell.decline_dispatch_op(context, 'Channel is invalidated');
-            return;
-        }
-
         if (chanType == Tp.IFACE_CHANNEL_TYPE_TEXT)
             this._approveTextChannel(account, conn, channel, dispatchOp, context);
         else if (chanType == Tp.IFACE_CHANNEL_TYPE_CALL)
             this._approveCall(account, conn, channel, dispatchOp, context);
         else if (chanType == Tp.IFACE_CHANNEL_TYPE_FILE_TRANSFER)
             this._approveFileTransfer(account, conn, channel, dispatchOp, context);
-        else
-            Shell.decline_dispatch_op(context, 'Unsupported channel type');
     },
 
     _approveTextChannel: function(account, conn, channel, dispatchOp, context) {
@@ -378,8 +365,9 @@ const Client = new Lang.Class({
 
     _ensureSubscriptionSource: function() {
         if (this._subscriptionSource == null) {
-            this._subscriptionSource = new MultiNotificationSource(
-                _("Subscription request"), 'gtk-dialog-question');
+            this._subscriptionSource = new MessageTray.Source(_("Subscription request"),
+                                                              'gtk-dialog-question',
+                                                              St.IconType.FULLCOLOR);
             Main.messageTray.add(this._subscriptionSource);
             this._subscriptionSource.connect('destroy', Lang.bind(this, function () {
                 this._subscriptionSource = null;
@@ -414,8 +402,9 @@ const Client = new Lang.Class({
 
     _ensureAccountSource: function() {
         if (this._accountSource == null) {
-            this._accountSource = new MultiNotificationSource(
-                _("Connection error"), 'gtk-dialog-error');
+            this._accountSource = new MessageTray.Source(_("Connection error"),
+                                                         'gtk-dialog-error',
+                                                         St.IconType.FULLCOLOR);
             Main.messageTray.add(this._accountSource);
             this._accountSource.connect('destroy', Lang.bind(this, function () {
                 this._accountSource = null;
@@ -431,14 +420,13 @@ const ChatSource = new Lang.Class({
     Extends: MessageTray.Source,
 
     _init: function(account, conn, channel, contact, client) {
-        this.parent(contact.get_alias());
-
-        this.isChat = true;
-
         this._account = account;
         this._contact = contact;
         this._client = client;
 
+        this.parent(contact.get_alias());
+
+        this.isChat = true;
         this._pendingMessages = [];
 
         this._conn = conn;
@@ -459,8 +447,6 @@ const ChatSource = new Lang.Class({
         this._receivedId = this._channel.connect('message-received', Lang.bind(this, this._messageReceived));
         this._pendingId = this._channel.connect('pending-message-removed', Lang.bind(this, this._pendingRemoved));
 
-        this._setSummaryIcon(this.createNotificationIcon());
-
         this._notifyAliasId = this._contact.connect('notify::alias', Lang.bind(this, this._updateAlias));
         this._notifyAvatarId = this._contact.connect('notify::avatar-file', Lang.bind(this, this._updateAvatarIcon));
         this._presenceChangedId = this._contact.connect('presence-changed', Lang.bind(this, this._presenceChanged));
@@ -523,10 +509,10 @@ const ChatSource = new Lang.Class({
     _getLogMessages: function() {
         let logManager = Tpl.LogManager.dup_singleton();
         let entity = Tpl.Entity.new_from_tp_contact(this._contact, Tpl.EntityType.CONTACT);
-        Shell.get_contact_events(logManager,
-                                 this._account, entity,
-                                 SCROLLBACK_HISTORY_LINES,
-                                 Lang.bind(this, this._displayPendingMessages));
+
+        logManager.get_filtered_events_async(this._account, entity,
+                                             Tpl.EventTypeMask.TEXT, SCROLLBACK_HISTORY_LINES,
+                                             null, Lang.bind(this, this._displayPendingMessages));
     },
 
     _displayPendingMessages: function(logManager, result) {
@@ -1015,10 +1001,9 @@ const ApproverSource = new Lang.Class({
     Extends: MessageTray.Source,
 
     _init: function(dispatchOp, text, gicon) {
-        this.parent(text);
-
         this._gicon = gicon;
-        this._setSummaryIcon(this.createNotificationIcon());
+
+        this.parent(text);
 
         this._dispatchOp = dispatchOp;
 
@@ -1041,7 +1026,6 @@ const ApproverSource = new Lang.Class({
 
     createNotificationIcon: function() {
         return new St.Icon({ gicon: this._gicon,
-                             icon_type: St.IconType.FULLCOLOR,
                              icon_size: this.ICON_SIZE });
     }
 });
@@ -1164,40 +1148,6 @@ const FileTransferNotification = new Lang.Class({
     }
 });
 
-// A notification source that can embed multiple notifications
-const MultiNotificationSource = new Lang.Class({
-    Name: 'MultiNotificationSource',
-    Extends: MessageTray.Source,
-
-    _init: function(title, icon) {
-        this.parent(title);
-
-        this._icon = icon;
-        this._setSummaryIcon(this.createNotificationIcon());
-        this._nbNotifications = 0;
-    },
-
-    notify: function(notification) {
-        this.parent(notification);
-
-        this._nbNotifications += 1;
-
-        // Display the source while there is at least one notification
-        notification.connect('destroy', Lang.bind(this, function () {
-            this._nbNotifications -= 1;
-
-            if (this._nbNotifications == 0)
-                this.destroy();
-        }));
-    },
-
-    createNotificationIcon: function() {
-        return new St.Icon({ gicon: Gio.icon_new_for_string(this._icon),
-                             icon_type: St.IconType.FULLCOLOR,
-                             icon_size: this.ICON_SIZE });
-    }
-});
-
 // Subscription request
 const SubscriptionRequestNotification = new Lang.Class({
     Name: 'SubscriptionRequestNotification',
