@@ -1,6 +1,7 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const AccountsService = imports.gi.AccountsService;
+const GdmGreeter = imports.gi.GdmGreeter;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const Lang = imports.lang;
@@ -473,13 +474,22 @@ const UserMenuButton = new Lang.Class({
                                        style_class: 'popup-menu-icon' });
         this._idleIcon = new St.Icon({ icon_name: 'user-idle',
                                        style_class: 'popup-menu-icon' });
+        this._pendingIcon = new St.Icon({ icon_name: 'user-status-pending',
+                                          style_class: 'popup-menu-icon' });
 
         this._accountMgr.connect('most-available-presence-changed',
                                   Lang.bind(this, this._updatePresenceIcon));
+        this._accountMgr.connect('account-enabled',
+                                  Lang.bind(this, this._onAccountEnabled));
+        this._accountMgr.connect('account-disabled',
+                                 Lang.bind(this, this._onAccountDisabled));
+        this._accountMgr.connect('account-removed',
+                                  Lang.bind(this, this._onAccountDisabled));
         this._accountMgr.prepare_async(null, Lang.bind(this,
             function(mgr) {
                 let [presence, s, msg] = mgr.get_most_available_presence();
                 this._updatePresenceIcon(mgr, presence, s, msg);
+                this._setupAccounts();
             }));
 
         this._name = new St.Label();
@@ -497,13 +507,13 @@ const UserMenuButton = new Lang.Class({
         }));
 
         this._userManager.connect('notify::is-loaded',
-                                  Lang.bind(this, this._updateSwitchUser));
+                                  Lang.bind(this, this._updateMultiUser));
         this._userManager.connect('notify::has-multiple-users',
-                                  Lang.bind(this, this._updateSwitchUser));
+                                  Lang.bind(this, this._updateMultiUser));
         this._userManager.connect('user-added',
-                                  Lang.bind(this, this._updateSwitchUser));
+                                  Lang.bind(this, this._updateMultiUser));
         this._userManager.connect('user-removed',
-                                  Lang.bind(this, this._updateSwitchUser));
+                                  Lang.bind(this, this._updateMultiUser));
         this._lockdownSettings.connect('changed::' + DISABLE_USER_SWITCH_KEY,
                                        Lang.bind(this, this._updateSwitchUser));
         this._lockdownSettings.connect('changed::' + DISABLE_LOG_OUT_KEY,
@@ -542,30 +552,32 @@ const UserMenuButton = new Lang.Class({
             this._name.set_text("");
     },
 
+    _updateMultiUser: function() {
+        this._updateSwitchUser();
+        this._updateLogout();
+    },
+
     _updateSwitchUser: function() {
         let allowSwitch = !this._lockdownSettings.get_boolean(DISABLE_USER_SWITCH_KEY);
-        if (allowSwitch &&
-            this._userManager.can_switch() &&
-            this._userManager.has_multiple_users)
-            this._loginScreenItem.actor.show();
-        else
-            this._loginScreenItem.actor.hide();
+        let multiUser = this._userManager.can_switch() && this._userManager.has_multiple_users;
+        let multiSession = GdmGreeter.get_session_ids().length > 1;
+
+        this._loginScreenItem.label.set_text(multiUser ? _("Switch User")
+                                                       : _("Switch Session"));
+        this._loginScreenItem.actor.visible = allowSwitch && (multiUser || multiSession);
     },
 
     _updateLogout: function() {
         let allowLogout = !this._lockdownSettings.get_boolean(DISABLE_LOG_OUT_KEY);
-        if (allowLogout)
-            this._logoutItem.actor.show();
-        else
-            this._logoutItem.actor.hide();
+        let multiUser = this._userManager.has_multiple_users;
+        let multiSession = GdmGreeter.get_session_ids().length > 1;
+
+        this._logoutItem.actor.visible = allowLogout && (multiUser || multiSession);
     },
 
     _updateLockScreen: function() {
         let allowLockScreen = !this._lockdownSettings.get_boolean(DISABLE_LOCK_SCREEN_KEY);
-        if (allowLockScreen)
-            this._lockScreenItem.actor.show();
-        else
-            this._lockScreenItem.actor.hide();
+        this._lockScreenItem.actor.visible = allowLockScreen;
     },
 
     _updateHaveShutdown: function() {
@@ -584,19 +596,16 @@ const UserMenuButton = new Lang.Class({
         if (!this._suspendOrPowerOffItem)
             return;
 
-        if (!this._haveShutdown && !this._haveSuspend)
-            this._suspendOrPowerOffItem.actor.hide();
-        else
-            this._suspendOrPowerOffItem.actor.show();
+        this._suspendOrPowerOffItem.actor.visible = this._haveShutdown || this._haveSuspend;
 
-        // If we can't suspend show Power Off... instead
+        // If we can't power off show Suspend instead
         // and disable the alt key
-        if (!this._haveSuspend) {
-            this._suspendOrPowerOffItem.updateText(_("Power Off..."), null);
-        } else if (!this._haveShutdown) {
+        if (!this._haveShutdown) {
             this._suspendOrPowerOffItem.updateText(_("Suspend"), null);
+        } else if (!this._haveSuspend) {
+            this._suspendOrPowerOffItem.updateText(_("Power Off"), null);
         } else {
-            this._suspendOrPowerOffItem.updateText(_("Suspend"), _("Power Off..."));
+            this._suspendOrPowerOffItem.updateText(_("Power Off"), _("Suspend"));
         }
     },
 
@@ -620,11 +629,52 @@ const UserMenuButton = new Lang.Class({
             this._iconBox.child = this._offlineIcon;
     },
 
+    _setupAccounts: function() {
+        let accounts = this._accountMgr.get_valid_accounts();
+        for (let i = 0; i < accounts.length; i++) {
+            accounts[i]._changingId = accounts[i].connect('notify::connection-status',
+                                                          Lang.bind(this, this._updateChangingPresence));
+        }
+        this._updateChangingPresence();
+    },
+
+    _onAccountEnabled: function(accountMgr, account) {
+        if (!account._changingId)
+            account._changingId = account.connect('notify::connection-status',
+                                                  Lang.bind(this, this._updateChangingPresence));
+        this._updateChangingPresence();
+    },
+
+    _onAccountDisabled: function(accountMgr, account) {
+        account.disconnect(account._changingId);
+        account._changingId = 0;
+        this._updateChangingPresence();
+    },
+
+    _updateChangingPresence: function() {
+        let accounts = this._accountMgr.get_valid_accounts();
+        let changing = false;
+        for (let i = 0; i < accounts.length; i++) {
+            if (accounts[i].connection_status == Tp.ConnectionStatus.CONNECTING) {
+                changing = true;
+                break;
+            }
+        }
+
+        if (changing) {
+            this._iconBox.child = this._pendingIcon;
+        } else {
+            let [presence, s, msg] = this._accountMgr.get_most_available_presence();
+            this._updatePresenceIcon(this._accountMgr, presence, s, msg);
+        }
+    },
+
     _createSubMenu: function() {
         let item;
 
         item = new IMStatusChooserItem();
-        item.connect('activate', Lang.bind(this, this._onMyAccountActivate));
+        if (Main.sessionMode.allowSettings)
+            item.connect('activate', Lang.bind(this, this._onMyAccountActivate));
         this.menu.addMenuItem(item);
         this._statusChooser = item;
 
@@ -636,28 +686,28 @@ const UserMenuButton = new Lang.Class({
         item = new PopupMenu.PopupSeparatorMenuItem();
         this.menu.addMenuItem(item);
 
-        item = new PopupMenu.PopupMenuItem(_("Online Accounts"));
-        item.connect('activate', Lang.bind(this, this._onOnlineAccountsActivate));
-        this.menu.addMenuItem(item);
+        if (Main.sessionMode.allowSettings) {
+            item = new PopupMenu.PopupMenuItem(_("System Settings"));
+            item.connect('activate', Lang.bind(this, this._onPreferencesActivate));
+            this.menu.addMenuItem(item);
+        }
 
-        item = new PopupMenu.PopupMenuItem(_("System Settings"));
-        item.connect('activate', Lang.bind(this, this._onPreferencesActivate));
+        item = new PopupMenu.PopupAlternatingMenuItem(_("Power Off"),
+                                                      _("Suspend"));
         this.menu.addMenuItem(item);
+        item.connect('activate', Lang.bind(this, this._onSuspendOrPowerOffActivate));
+        this._suspendOrPowerOffItem = item;
+        this._updateSuspendOrPowerOff();
 
         item = new PopupMenu.PopupSeparatorMenuItem();
         this.menu.addMenuItem(item);
 
-        item = new PopupMenu.PopupMenuItem(_("Lock Screen"));
-        item.connect('activate', Lang.bind(this, this._onLockScreenActivate));
-        this.menu.addMenuItem(item);
-        this._lockScreenItem = item;
-
         item = new PopupMenu.PopupMenuItem(_("Switch User"));
         item.connect('activate', Lang.bind(this, this._onLoginScreenActivate));
         this.menu.addMenuItem(item);
         this._loginScreenItem = item;
 
-        item = new PopupMenu.PopupMenuItem(_("Log Out..."));
+        item = new PopupMenu.PopupMenuItem(_("Log Out"));
         item.connect('activate', Lang.bind(this, this._onQuitSessionActivate));
         this.menu.addMenuItem(item);
         this._logoutItem = item;
@@ -665,12 +715,10 @@ const UserMenuButton = new Lang.Class({
         item = new PopupMenu.PopupSeparatorMenuItem();
         this.menu.addMenuItem(item);
 
-        item = new PopupMenu.PopupAlternatingMenuItem(_("Suspend"),
-                                                      _("Power Off..."));
+        item = new PopupMenu.PopupMenuItem(_("Lock"));
+        item.connect('activate', Lang.bind(this, this._onLockScreenActivate));
         this.menu.addMenuItem(item);
-        this._suspendOrPowerOffItem = item;
-        item.connect('activate', Lang.bind(this, this._onSuspendOrPowerOffActivate));
-        this._updateSuspendOrPowerOff();
+        this._lockScreenItem = item;
     },
 
     _updatePresenceStatus: function(item, event) {
@@ -698,12 +746,6 @@ const UserMenuButton = new Lang.Class({
         app.activate();
     },
 
-    _onOnlineAccountsActivate: function() {
-        Main.overview.hide();
-        let app = Shell.AppSystem.get_default().lookup_setting('gnome-online-accounts-panel.desktop');
-        app.activate(-1);
-    },
-
     _onPreferencesActivate: function() {
         Main.overview.hide();
         let app = Shell.AppSystem.get_default().lookup_app('gnome-control-center.desktop');
@@ -732,14 +774,14 @@ const UserMenuButton = new Lang.Class({
     _onSuspendOrPowerOffActivate: function() {
         Main.overview.hide();
 
-        if (this._haveSuspend &&
+        if (this._haveShutdown &&
             this._suspendOrPowerOffItem.state == PopupMenu.PopupAlternatingMenuItemState.DEFAULT) {
+            this._session.ShutdownRemote();
+        } else {
             // Ensure we only suspend after locking the screen
             this._screenSaverProxy.LockRemote(Lang.bind(this, function() {
                 this._upClient.suspend_sync(null);
             }));
-        } else {
-            this._session.ShutdownRemote();
         }
     }
 });
