@@ -528,7 +528,7 @@ const WindowOverlay = new Lang.Class({
 
         let settings = new Gio.Settings({ schema: BUTTON_LAYOUT_SCHEMA });
         let layout = settings.get_string(BUTTON_LAYOUT_KEY);
-        let rtl = St.Widget.get_default_direction() == St.TextDirection.RTL;
+        let rtl = Clutter.get_default_text_direction() == Clutter.TextDirection.RTL;
 
         let split = layout.split(":");
         let side;
@@ -1134,7 +1134,6 @@ const Workspace = new Lang.Class({
     _hideAllOverlays: function() {
         for (let i = 0; i < this._windows.length; i++) {
             let clone = this._windows[i];
-            Tweener.removeTweens(clone.actor);
             let overlay = this._windowOverlays[i];
             if (overlay)
                 overlay.hide();
@@ -1252,7 +1251,7 @@ const Workspace = new Lang.Class({
         if (!this._isMyWindow(win) || !this._isOverviewWindow(win))
             return;
 
-        let clone = this._addWindowClone(win);
+        let [clone, overlay] = this._addWindowClone(win);
 
         if (win._overviewHint) {
             let x = win._overviewHint.x - this.actor.x;
@@ -1262,6 +1261,7 @@ const Workspace = new Lang.Class({
 
             clone.actor.set_position (x, y);
             clone.actor.set_scale (scale, scale);
+            this._updateWindowOverlayPositions(clone, overlay, x, y, scale, false);
         } else {
             // Position new windows at the top corner of the workspace rather
             // than where they were placed for real to avoid the window
@@ -1321,7 +1321,10 @@ const Workspace = new Lang.Class({
 
         this.leavingOverview = true;
 
-        this._hideAllOverlays();
+        for (let i = 0; i < this._windows.length; i++) {
+            let clone = this._windows[i];
+            Tweener.removeTweens(clone.actor);
+        }
 
         if (this._repositionWindowsId > 0) {
             Mainloop.source_remove(this._repositionWindowsId);
@@ -1361,6 +1364,7 @@ const Workspace = new Lang.Class({
             }
         }
 
+        this._hideAllOverlays();
     },
 
     destroy : function() {
@@ -1451,7 +1455,7 @@ const Workspace = new Lang.Class({
         this._windows.push(clone);
         this._windowOverlays.push(overlay);
 
-        return clone;
+        return [clone, overlay];
     },
 
     _onShowOverlayClose: function (windowOverlay) {
