@@ -28,6 +28,8 @@
 #include "shell-perf-log.h"
 #include "st.h"
 
+#include <jsapi.h>
+
 extern GType gnome_shell_plugin_get_type (void);
 
 #define SHELL_DBUS_SERVICE "org.gnome.Shell"
@@ -36,6 +38,7 @@ extern GType gnome_shell_plugin_get_type (void);
 #define OVERRIDES_SCHEMA "org.gnome.shell.overrides"
 
 static gboolean is_gdm_mode = FALSE;
+static char *session_mode = NULL;
 
 #define DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER 1
 #define DBUS_REQUEST_NAME_REPLY_ALREADY_OWNER 4
@@ -174,6 +177,18 @@ shell_prefs_init (void)
                                          OVERRIDES_SCHEMA);
 }
 
+static void
+shell_introspection_init (void)
+{
+
+  g_irepository_prepend_search_path (MUTTER_TYPELIB_DIR);
+  g_irepository_prepend_search_path (GNOME_SHELL_PKGLIBDIR);
+#if HAVE_BLUETOOTH
+  g_irepository_prepend_search_path (BLUETOOTH_DIR);
+#endif
+
+}
+
 static void
 malloc_statistics_callback (ShellPerfLog *perf_log,
                             gpointer      data)
@@ -239,6 +254,48 @@ default_log_handler (const char     *log_domain,
     g_log_default_handler (log_domain, log_level, message, data);
 }
 
+static void
+shut_up (const char     *domain,
+         GLogLevelFlags  level,
+         const char     *message,
+         gpointer        user_data)
+{
+}
+
+static gboolean
+list_modes (const char  *option_name,
+            const char  *value,
+            gpointer     data,
+            GError     **error)
+{
+  ShellGlobal *global;
+  GjsContext *context;
+  const char *script;
+  int status;
+
+  /* Many of our imports require global to be set, so rather than
+   * tayloring our imports carefully here to avoid that dependency,
+   * we just set it.
+   * ShellGlobal has some GTK+ dependencies, so initialize GTK+; we
+   * don't really care if it fails though (e.g. when running from a tty),
+   * so we mute all warnings */
+  g_log_set_default_handler (shut_up, NULL);
+  gtk_init_check (NULL, NULL);
+
+  _shell_global_init (NULL);
+  global = shell_global_get ();
+  context = _shell_global_get_gjs_context (global);
+
+  shell_introspection_init ();
+
+  script = "imports.ui.environment.init();"
+           "imports.ui.sessionMode.listModes();";
+  if (!gjs_context_eval (context, script, -1, "<main>", &status, NULL))
+      g_message ("Retrieving list of available modes failed.");
+
+  exit (status);
+}
+
 static gboolean
 print_version (const gchar    *option_name,
                const gchar    *value,
@@ -257,11 +314,23 @@ GOptionEntry gnome_shell_options[] = {
     NULL
   },
   {
-    "gdm-mode", 0, 0, G_OPTION_ARG_NONE,
+    "gdm-mode", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE,
     &is_gdm_mode,
     N_("Mode used by GDM for login screen"),
     NULL
   },
+  {
+    "mode", 0, 0, G_OPTION_ARG_STRING,
+    &session_mode,
+    N_("Use a specific mode, e.g. \"gdm\" for login screen"),
+    "MODE"
+  },
+  {
+    "list-modes", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
+    list_modes,
+    N_("List possible modes"),
+    NULL
+  },
   { NULL }
 };
 
@@ -270,7 +339,6 @@ main (int argc, char **argv)
 {
   GOptionContext *ctx;
   GError *error = NULL;
-  ShellSessionType session_type;
   int ecode;
   TpDebugSender *sender;
 
@@ -309,11 +377,7 @@ main (int argc, char **argv)
   shell_a11y_init ();
   shell_perf_log_init ();
   shell_prefs_init ();
-
-  g_irepository_prepend_search_path (GNOME_SHELL_PKGLIBDIR);
-#if HAVE_BLUETOOTH
-  g_irepository_prepend_search_path (BLUETOOTH_DIR);
-#endif
+  shell_introspection_init ();
 
   /* Turn on telepathy-glib debugging but filter it out in
    * default_log_handler. This handler also exposes all the logs over D-Bus
@@ -324,12 +388,10 @@ main (int argc, char **argv)
   g_log_set_default_handler (default_log_handler, sender);
 
   /* Initialize the global object */
-  if (is_gdm_mode)
-      session_type = SHELL_SESSION_GDM;
-  else
-      session_type = SHELL_SESSION_USER;
+  if (session_mode == NULL)
+    session_mode = is_gdm_mode ? "gdm" : "user";
 
-  _shell_global_init ("session-type", session_type, NULL);
+  _shell_global_init ("session-mode", session_mode, NULL);
 
   ecode = meta_run ();
 
@@ -350,7 +412,6 @@ main (int argc, char **argv)
    This function is never actually called.
    https://bugzilla.gnome.org/show_bug.cgi?id=670477
 */
-
 void _shell_link_to_shell_js (void);
 
 void
