@@ -59,7 +59,7 @@ struct _ShellGlobal {
   MetaScreen *meta_screen;
   GdkScreen *gdk_screen;
 
-  ShellSessionType session_type;
+  char *session_mode;
 
   /* We use this window to get a notification from GTK+ when
    * a widget in our process does a GTK+ grab.  See
@@ -97,7 +97,7 @@ struct _ShellGlobal {
 enum {
   PROP_0,
 
-  PROP_SESSION_TYPE,
+  PROP_SESSION_MODE,
   PROP_OVERLAY_GROUP,
   PROP_SCREEN,
   PROP_GDK_SCREEN,
@@ -143,8 +143,9 @@ shell_global_set_property(GObject         *object,
     case PROP_STAGE_INPUT_MODE:
       shell_global_set_stage_input_mode (global, g_value_get_enum (value));
       break;
-    case PROP_SESSION_TYPE:
-      global->session_type = g_value_get_enum (value);
+    case PROP_SESSION_MODE:
+      g_clear_pointer (&global->session_mode, g_free);
+      global->session_mode = g_ascii_strdown (g_value_get_string (value), -1);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -162,8 +163,8 @@ shell_global_get_property(GObject         *object,
 
   switch (prop_id)
     {
-    case PROP_SESSION_TYPE:
-      g_value_set_enum (value, shell_global_get_session_type (global));
+    case PROP_SESSION_MODE:
+      g_value_set_string (value, shell_global_get_session_mode (global));
       break;
     case PROP_OVERLAY_GROUP:
       g_value_set_object (value, meta_get_overlay_group_for_screen (global->meta_screen));
@@ -343,13 +344,12 @@ shell_global_class_init (ShellGlobalClass *klass)
                     G_TYPE_STRING);
 
   g_object_class_install_property (gobject_class,
-                                   PROP_SESSION_TYPE,
-                                   g_param_spec_enum ("session-type",
-                                                      "Session Type",
-                                                      "The type of session",
-                                                      SHELL_TYPE_SESSION_TYPE,
-                                                      SHELL_SESSION_USER,
-                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+                                   PROP_SESSION_MODE,
+                                   g_param_spec_string ("session-mode",
+                                                        "Session Mode",
+                                                        "The session mode to use",
+                                                        "user",
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property (gobject_class,
                                    PROP_OVERLAY_GROUP,
                                    g_param_spec_object ("overlay-group",
@@ -805,90 +805,9 @@ global_stage_after_paint (ClutterStage *stage,
                         "clutter.stagePaintDone");
 }
 
-static void
-update_font_options (GtkSettings  *settings,
-                     ClutterStage *stage)
-{
-  StThemeContext *context;
-  ClutterBackend *backend;
-  gint dpi;
-  gint hinting;
-  gchar *hint_style_str;
-  cairo_hint_style_t hint_style = CAIRO_HINT_STYLE_NONE;
-  gint antialias;
-  cairo_antialias_t antialias_mode = CAIRO_ANTIALIAS_NONE;
-  cairo_font_options_t *options;
-
-  g_object_get (settings,
-                "gtk-xft-dpi", &dpi,
-                "gtk-xft-antialias", &antialias,
-                "gtk-xft-hinting", &hinting,
-                "gtk-xft-hintstyle", &hint_style_str,
-                NULL);
-
-  context = st_theme_context_get_for_stage (stage);
-
-  if (dpi != -1)
-    /* GTK stores resolution as 1024 * dots/inch */
-    st_theme_context_set_resolution (context, dpi / 1024);
-  else
-    st_theme_context_set_default_resolution (context);
-
-  /* Clutter (as of 0.9) passes comprehensively wrong font options
-   * override whatever set_font_flags() did above.
-   *
-   * http://bugzilla.openedhand.com/show_bug.cgi?id=1456
-   */
-  backend = clutter_get_default_backend ();
-  options = cairo_font_options_create ();
-
-  cairo_font_options_set_hint_metrics (options, CAIRO_HINT_METRICS_ON);
-
-  if (hinting >= 0 && !hinting)
-    {
-      hint_style = CAIRO_HINT_STYLE_NONE;
-    }
-  else if (hint_style_str)
-    {
-      if (strcmp (hint_style_str, "hintnone") == 0)
-        hint_style = CAIRO_HINT_STYLE_NONE;
-      else if (strcmp (hint_style_str, "hintslight") == 0)
-        hint_style = CAIRO_HINT_STYLE_SLIGHT;
-      else if (strcmp (hint_style_str, "hintmedium") == 0)
-        hint_style = CAIRO_HINT_STYLE_MEDIUM;
-      else if (strcmp (hint_style_str, "hintfull") == 0)
-        hint_style = CAIRO_HINT_STYLE_FULL;
-    }
-
-  g_free (hint_style_str);
-
-  cairo_font_options_set_hint_style (options, hint_style);
-
-  /* We don't want to turn on subpixel anti-aliasing; since Clutter
-   * doesn't currently have the code to support ARGB masks,
-   * generating them then squashing them back to A8 is pointless.
-   */
-  antialias_mode = (antialias < 0 || antialias) ? CAIRO_ANTIALIAS_GRAY
-                                                : CAIRO_ANTIALIAS_NONE;
-
-  cairo_font_options_set_antialias (options, antialias_mode);
-
-  clutter_backend_set_font_options (backend, options);
-  cairo_font_options_destroy (options);
-}
-
-static void
-settings_notify_cb (GtkSettings *settings,
-                    GParamSpec  *pspec,
-                    gpointer     data)
-{
-  update_font_options (settings, CLUTTER_STAGE (data));
-}
-
 static void
 shell_fonts_init (ClutterStage *stage)
 {
-  GtkSettings *settings;
   CoglPangoFontMap *fontmap;
 
   /* Disable text mipmapping; it causes problems on pre-GEM Intel
@@ -898,19 +817,6 @@ shell_fonts_init (ClutterStage *stage)
    */
   fontmap = COGL_PANGO_FONT_MAP (clutter_get_font_map ());
   cogl_pango_font_map_set_use_mipmapping (fontmap, FALSE);
-
-  settings = gtk_settings_get_default ();
-  g_object_connect (settings,
-                    "signal::notify::gtk-xft-dpi",
-                    G_CALLBACK (settings_notify_cb), stage,
-                    "signal::notify::gtk-xft-antialias",
-                    G_CALLBACK (settings_notify_cb), stage,
-                    "signal::notify::gtk-xft-hinting",
-                    G_CALLBACK (settings_notify_cb), stage,
-                    "signal::notify::gtk-xft-hintstyle",
-                    G_CALLBACK (settings_notify_cb), stage,
-                    NULL);
-  update_font_options (settings, stage);
 }
 
 /* This is an IBus workaround. The flow of events with IBus is that every time
@@ -1870,31 +1776,10 @@ shell_global_launch_calendar_server (ShellGlobal *global)
   g_free (calendar_server_exe);
 }
 
-/**
- * shell_global_get_session_type:
- * @global: The #ShellGlobal.
- *
- * Gets the type of session gnome-shell provides.
- *
- * The type determines what UI elements are displayed,
- * what keybindings work, and generally how the shell
- * behaves.
- *
- * A session type of #SHELL_SESSION_USER means gnome-shell
- * will enable the activities overview, status menu, run dialog,
- * etc. This is the default.
- *
- * A session type of #SHELL_SESSION_GDM means gnome-shell
- * will enable a login dialog and run in a more confined
- * way. This type is suitable for the display manager.
- *
- * Returns: the type of session gnome-shell is providing.
- */
-ShellSessionType
-shell_global_get_session_type (ShellGlobal  *global)
+const char *
+shell_global_get_session_mode (ShellGlobal *global)
 {
-  g_return_val_if_fail (SHELL_IS_GLOBAL (global),
-                        SHELL_SESSION_USER);
+  g_return_val_if_fail (SHELL_IS_GLOBAL (global), "user");
 
-  return global->session_type;
+  return global->session_mode;
 }
