@@ -141,12 +141,7 @@ void     shell_global_reexec_self               (ShellGlobal  *global);
 
 void     shell_global_launch_calendar_server    (ShellGlobal  *global);
 
-typedef enum {
-  SHELL_SESSION_USER,
-  SHELL_SESSION_GDM
-} ShellSessionType;
-
-ShellSessionType shell_global_get_session_type  (ShellGlobal  *global);
+const char *     shell_global_get_session_mode  (ShellGlobal  *global);
 
 G_END_DECLS
 
